# -*- coding: utf-8 -*-
# _l1norm.py
# Module providing the l1norm function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the l1norm() function
"""

from __future__ import division
import numpy as np
from scipy.signal import dimpulse

def l1norm(H):
	"""Compute the l1-norm of a z-domain transfer function.
	"""
	_, y = dimpulse(H, t=np.arange(100))
	return np.sum(np.abs(y[0]))

def test_l1norm():
	"""Test function for l1norm()
	"""
	zeros = np.array(())
	poles = np.array((.5,))
	k = 1.
	zpkt_tuple = zeros, poles, k, 1
	assert np.allclose(l1norm(zpkt_tuple), 2., rtol=1e-5, atol=1e-8)




# -*- coding: utf-8 -*-
# _dbv.py
# This module provides the dbv function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbv() function, used to convert a voltage gain to dB.
"""

import numpy as np
from ._undbv import undbv
import numpy as np

from ._utils import carray, save_input_form, restore_input_form

def dbv(x):
	"""The dB equivalent of the voltage ratio ``x``.

	.. math::

	    G_{dB} = 20 \\mathrm{log}_{10}(|x|)

	**Parameters:**

	x : scalar or sequence
	    The voltage (ratio) to be converted.

	**Returns:**

	GdB : scalar or sequence
	       The input voltage (ratio) expressed in dB.

	.. seealso:: :func:`undbv`, :func:`db`, :func:`dbp`, :func:`dbm`

	"""
	iform = save_input_form(x)
	x = carray(x)
	y = -np.inf*np.ones(x.shape)
	nonzero = (x != 0)
	y[nonzero] = 20.*np.log10(np.abs(x[nonzero]))
	return restore_input_form(y, iform)

def test_dbv():
	"""Test function for dbv()"""
	t = np.array([3.0])
	r1 = undbv(dbv(t))
	assert np.allclose(t, r1, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _undbm.py
# This module provides the undbm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the undbm() function.
"""

import numpy as np

from ._utils import carray, save_input_form, restore_input_form

def undbm(p, z=50):
	"""Calculate the RMS voltage equivalent of a power ``p`` expressed in dBm.

	.. math::

	    V_{\\mathrm{RMS}} = \\sqrt{z\\ 10^{p/10 - 3}}

	**Parameters:**

	p : scalar or sequence
	    The power to be converted.

	z : scalar, optional
	    The normalization resistance value, defaults to 50 ohm.

	**Returns:**

	Vrms : scalar or sequence
	       The RMS voltage corresponding to p

	.. seealso:: :func:`undbp`, :func:`undbv`, :func:`dbm`, :func:`db`

	"""
	iform = save_input_form(p)
	p = carray(p)
	up = np.sqrt(z*10.**(p/10.-3))
	return restore_input_form(up, iform)

def test():
	"""Test function for undbm()"""
	assert np.allclose([undbm(53.015)], [100.054125892], rtol=1e-05, atol=1e-08)
	assert np.allclose([undbm(3, 100)], [0.44668359215], rtol=1e-05, atol=1e-08)
	assert np.isscalar(undbm(3, 100))




# -*- coding: utf-8 -*-
# _ds_synNTFobj1.py
# Module providing the ds_synNTFobj1 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the ds_synNTFobj1() function
"""

import numpy as np

from ._padt import padt
from ._padb import padb 
from ._db import db
from ._ds_f1f2 import ds_f1f2 
from ._utils import zpk, carray
from ._rmsGain import rmsGain

def ds_synNTFobj1(x, p, osr, f0):
	"""Objective function for :func:`synthesizeNTF`

	This function is not meant to be used directly but it is provided for compliance with the 
	MATLAB DS Toolbox.

	"""
	p = carray(p)
	z = np.exp(2j*np.pi*(f0 + 0.5/osr*x))
	z = carray(z)
	if f0 > 0:
		z = padt(z, p.shape[0]/2., np.exp(2j*np.pi*f0))

	z = np.hstack((z, np.conj(z))) 
	z = z[:]
	if f0 == 0:
		z = padb(z, p.shape[0], 1)

	f1, f2 = ds_f1f2(osr, f0)
	ntf = zpk(z, p, 1)
	y = db(rmsGain(ntf, f1, f2))
	return y

def test_ds_synNTFobj1():
	"""Test function for ds_synNTFobj1()
	"""
	res = -27.167735573627283
	tv =  ds_synNTFobj1(.5, (.9, 2), 64, .1)
	np.allclose((res,), (tv, ), atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _axisLabels.py
# Module providing the axisLabel function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the axisLabel() function
"""

import numpy as np

def axisLabels(ran, incr):
	"""Utility function to quickly generate the alphanum. axis labels.

	**Parameters:**

	ran : sequence
	    Sequence containing the axis points (floats)

	incr : int, or 2-elements sequence
	    This parameter may be:

	* an int, the function returns an array of strings corresponding to:
	  each element of ``range[0]:range[-1]:incr`` formatted as ``'%g'``.

	* a list, the function returns an array of strings corresponding to:
	  each element of ``incr[1]:range[-1]:incr[0]`` formatted as ``'%g'``.
	
	.. note:: All elements in `ran` less than 1e-6 are rounded down to 0.

	**Returns:**

	labels : list of strings

	**Raises:**

	ValueError: "Unrecognised incr."
	"""
	ran[np.abs(ran)<1e-6] = 0
	s = []
	if not hasattr(incr, '__len__'):
		incr = int(incr)
		first = 0
	elif len(incr) == 2:
		first = incr[1]
		incr = incr[0]
	else:
		raise ValueError("Unrecognised incr: "+str(incr))
	for i in range(first, len(ran), incr):
		s += ['%g' % ran[i]]
	return s
	
def test_axisLabels():
	"""Test function for axisLabels()
	"""
	ran = np.arange(100)
	ss = axisLabels(ran, incr=10)
	r = ['0', '10', '20', '30', '40', '50', '60', '70', '80', '90']
	assert r == ss




# -*- coding: utf-8 -*-
# _bplogsmooth.py
# Module providing the bplogsmooth function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bplogsmooth() function, which smooths an fft and 
converts it to dB.
"""

import numpy as np
from numpy.linalg import norm

from ._dbp import dbp
from ._utils import carray

def bplogsmooth(X, tbin, f0):
	"""Smooth the fft, X, and convert it to dB.

	Use 8 bins from the bin corresponding to f0 to tbin and again as far.
	Thereafter increase bin sizes by a factor of 1.1, staying less than 2^10.
	For tbin, group the bins together.

	Use this for nice double-sided log-log plots.

	.. note:: ``tbin`` is assumed to be in the upper sideband!

	.. seealso:: :func:`logsmooth`

	"""
	X = carray(X)
	if hasattr(X, 'shape') and len(X.shape) == 2 and \
	   not X.shape[0]*X.shape[1] == max(X.shape):
		raise ValueError("The X vector is not unidimensional: " + str(X.shape))
	N = max(X.shape)
	tbin = int(tbin)
	n = 8

	bin0 = np.round(f0*N, 0)
	bin1 = ((tbin - bin0) % n) + bin0
	bind = bin1 - bin0
	usb1 = np.concatenate((np.arange(bin1, tbin+1, n), 
	                       np.arange(tbin+3, tbin+bind+1, 8)
	                     ))
	m = usb1[-1] + n
	while m + n/2. < N/2.:
		usb1 = np.concatenate((usb1, np.array((m,))))
		n = min(n*1.1, 2**10)
		m = m + n
	usb2 = np.concatenate((usb1[1:]-1, np.array((N/2.,))))

	n = 8
	lsb2 = np.arange(bin1, bin1 - 2*bind + 1, -n) - 1
	m = lsb2[-1] - n
	while m - n/2. > 1:
		lsb2 = np.concatenate((lsb2, np.array((m,))))
		n = min(n*1.1, 2**10)
		m = m - n
	lsb1 = np.concatenate((lsb2[1:] + 1, np.ones((1,))))

	startbin = np.concatenate((lsb1[::-1], usb1)) - 1
	stopbin = np.concatenate((lsb2[::-1], usb2)) - 1

	f = ((startbin + stopbin)/2. - 1.)/N - f0
	p = np.zeros(f.shape)
	for i in range(f.shape[0]):
		p[i] = dbp(
		           norm(X[startbin[i]:stopbin[i]+1]**2. /
		                (stopbin[i] - startbin[i] + 1.)
		               )
		          )
	return f, p

def test_bplogsmooth():
	"""Test function for bplogsmooth()
	"""
	# FIXME WRITE PROPER TEST
	bplogsmooth(np.arange(200), 10, 0)




# -*- coding: utf-8 -*-
# _dbm.py
# This module provides the dbm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbm() function, used to convert an RMS voltage to dBm.
"""

from __future__ import division
import numpy as np

from ._utils import carray, save_input_form, restore_input_form

def dbm(v, R=50):
	"""The dBm equivalent of an RMS voltage ``v``.

	.. math::

	    P_{dBm} = 10 \\mathrm{log}_{10}(1000 \\frac{v^2}{R} )

	**Parameters:**

	v : scalar or sequence
	    The voltages to be converted.

	R : scalar, optional
	    The resistor value the power is calculated upon, defaults to 50 ohm.

	**Returns:**

	PdBm : scalar or sequence
	       The input in dBm.

	.. seealso:: :func:`undbm`, :func:`db`, :func:`dbp`, :func:`dbv`

	"""
	iform = save_input_form(v)
	v = carray(v)
	y = -np.Inf*np.ones(np.size(v))
	nonzero = (v != 0)
	y[nonzero] = 10.*np.log10(np.abs(v[nonzero]**2.)/R) + 30
	return restore_input_form(y, iform)

def test_dbm():
	"""Test function for dbm()
	"""
	v = np.arange(10)*1e-3
	r = [-np.inf, -46.98970004, -40.96910013, -37.44727495, -34.94850022,
	     -33.01029996, -31.42667504, -30.08773924, -28.9279003, -27.90484985]
	assert np.allclose(dbm(v), r, atol=1e-8, rtol=1e-5)
	v = 9e-3 # test scalars.
	r = -27.90484985
	assert np.allclose(dbm(v), r, atol=1e-8, rtol=1e-5)
	assert np.isscalar(dbm(v))



# -*- coding: utf-8 -*-
# _dbp.py
# This module provides the dbp function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbp() function, used to convert a power gain to dB.
"""

import numpy as np

from ._utils import carray, save_input_form, restore_input_form

def dbp(x):
	"""The dB equivalent of the power ratio ``x``.

	.. math::

	    P_{dB} = 10 \\mathrm{log}_{10}(x)

	**Parameters:**

	x : scalar or sequence
	    The power ratio to be converted.

	**Returns:**

	PdB : scalar or sequence
	       The input expressed in dB.

	.. seealso:: :func:`undbp`, :func:`db`, :func:`dbm`, :func:`dbv`

	"""
	iform = save_input_form(x)
	x = carray(x)
	y = -np.Inf*np.ones(x.shape)
	nonzero = (x != 0)
	y[nonzero] = 10.*np.log10(np.abs(x[nonzero]))
	return restore_input_form(y, iform)

def test_dbp():
	"""Test function for dbp()
	"""
	tv = np.array([2])
	r = np.array([3.01029996])
	res = dbp(tv)
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
	tv = 2
	r = 3.01029996
	res = dbp(tv)
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
	assert np.isscalar(res) # check for type coherence
	tv = 2, 2
	r = 3.01029996, 3.01029996
	res = dbp(tv)
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)



# -*- coding: utf-8 -*-
# _zinc.py
# The zinc function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the zinc() function which calculates the magnitude 
response of a cascade of comb filters.
"""

import numpy as np

def zinc(f, m=64, n=1):
	"""Calculate the magnitude response of a cascade of n mth-order 
	comb filters at frequencies f.
	"""
	return np.fabs(np.sinc(m*f)/np.sinc(f) )**n

def test_zinc():
	"""Test function for zinc()"""
	ref = [1.0000,    0.9985,    0.9941,    0.9867,    0.9765,    0.9635,
               0.9478,    0.9295,    0.9087,    0.8855,    0.8602,    0.8329,
               0.8038,    0.7730,    0.7408,    0.7074,    0.6729,    0.6377,
               0.6019,    0.5658,    0.5295,    0.4933,    0.4574,    0.4221,
               0.3874,    0.3536,    0.3208,    0.2892,    0.2590,    0.2302,
               0.2031,    0.1776,    0.1538,    0.1319,    0.1118,    0.0936,
               0.0772,    0.0626,    0.0499,    0.0389,    0.0295,    0.0217,
               0.0154,    0.0104,    0.0066,    0.0038,    0.0020,    0.0008,
               0.0002,    0.0000,    0.0000]
	f = np.arange(0, 0.51, 0.01)
	test = zinc(f, 2, 3)
	assert np.allclose(test, ref, atol=1e-4, rtol=1e-4)



# -*- coding: utf-8 -*-
# _bquantize.py
# Bipolar quantization module
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the bquantize() function, used to bidirectionally 
quantize a vector to signed digits.
"""

from __future__ import division, print_function
import numpy as np
from ._constants import eps
from ._utils import empty, mfloor

def bquantize(x, nsd=3, abstol=eps, reltol=10*eps):
	"""Bidirectionally quantize a (n by 1) vector ``x`` to ``nsd`` signed digits.

	This method will terminate early if the error is less than the specified
	tolerances.

	The quantizer details are repeted here for the user's convenience:

	    The quantizer is ideal, producing integer outputs centered about zero. Quantizers with an even
	    number of levels are of the mid-rise type and produce outputs which are odd integers. Quantizers
	    with an odd number of levels are of the mid-tread type and produce outputs which are even inte-
	    gers.

	    .. image:: ../doc/_static/quantizer_model.png
	        :align: center
	        :alt: Quantizer model

	**Parameters:**

	x : array_like
           the data to be quantized.

	nsd : int, optional
              The number of signed digits.

        abstol and reltol : floats, optional
              If not supplied, the absolute tolerance and the relative 
              tolerance default to ``eps`` and ``10*eps``, resp.

	**Returns:**

	y : list 
            List of objects described below.

	y is a list of instances with the same length as x and the 
	following attributes:

	* ``y[i].val`` is the quantized value in floating-point form,
	* ``y[i].csd`` is a 2-by-nsd (or less) matrix containing
	  the powers of two (first row) and their signs (second row).
	
	.. seealso:: 
	    :func:`bunquantize`

	"""

	n = x.shape[0] if hasattr(x, 'shape') else len(x)
	#q = np.zeros((2*n, nsd)) in the original source #rep?
	y = [empty() for i in range(n)]
	offset = -np.log2(0.75)

	for i in range(n):
		xp = x[i]
		y[i].val = 0.
		y[i].csd = np.zeros((2, 0), dtype='int16')
		for _ in range(nsd):
			error = np.abs(y[i].val - x[i])
			if error <= abstol and error <= np.abs(x[i])*reltol: #rep? in the orig: or
				break
			p = mfloor(np.log2(np.abs(xp)) + offset)
			p2 = 2**p
			sx = np.sign(xp)
			xp = xp - sx*p2
			y[i].val = y[i].val + sx*p2
			addme = np.array((p, sx)).reshape((2, 1))
			y[i].csd = np.concatenate((y[i].csd, addme), axis=1)
	return y
	
def test_bquantize():
	"""Test function for bquantize()
	"""
	import scipy.io, pkg_resources
	x = np.linspace(-10, 10, 101)
	y = bquantize(x)
	yval = [yi.val for yi in y]
	ycsd = [yi.csd for yi in y]
	fname = pkg_resources.resource_filename(__name__, "test_data/test_bquantize.mat")
	s = scipy.io.loadmat(fname)['s']
	mval = []
	mcsd = []
	for i in range(s.shape[1]):
		mval.append(float(s[0, i][0]))
		mcsd.append(s[0, i][1])
	for i in range(len(mval)):
		assert np.allclose(mval[i], yval[i], atol=1e-8, rtol=1e-5)
		print(mcsd[i].shape, ycsd[i].shape)
		assert np.prod(mcsd[i].shape) + np.prod(ycsd[i].shape) == 0 or \
		       mcsd[i].shape == ycsd[i].shape

		if not 0 in ycsd[i].shape:
			assert np.allclose(mcsd[i], ycsd[i], atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _pulse.py
# This module provides the pulse function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the pulse() function, which calculates the sampled 
pulse response of a CT system.
"""

from __future__ import division
import numpy as np
from scipy.signal import step2, lti

from ._utils import lcm, rat

def pulse(S, tp=(0., 1.), dt=1., tfinal=10., nosum=False):
	""" y = pulse(S, tp=[0 1], dt=1, tfinal=10, nosum=0)
	Calculate the sampled pulse response of a CT system. 
	tp may be an array of pulse timings, one for each input.
	Outputs:
	y	The pulse response
	
	Inputs
	S	A **list** of LTI objects specifying the system.
	tp	An (n, 2) array of pulse timings
	dt	The time increment
	tfinal	The time of the last desired sample
	nosum	A flag indicating that the responses are not to be summed
	"""
	tp = np.mat(np.array(tp))
	if len(tp.shape) == 1:
		if not tp.shape[0] == 2:
			raise ValueError("tp is not (n, 2)-shaped")
		tp.reshape((1, tp.shape[0]))
	if len(tp.shape) == 2: 
		if not tp.shape[1] == 2:
			raise ValueError("tp is not (n, 2)-shaped")

	# Compute the time increment
	dd = 1;
	for tpi in np.nditer(tp.T.copy(order='C')):
		x, di = rat(tpi, 1e-3)
		dd = lcm(di, dd)

	x, ddt = rat(dt, 1e-3)
	x, df = rat(tfinal, 1e-3)
	delta_t = 1. / lcm(dd, lcm(ddt, df))
	delta_t = max(1e-3, delta_t)	# Put a lower limit on delta_t
	y1 = None
	for Si in S:
		T1, y1i= step2(Si, T=np.arange(0., tfinal + delta_t, delta_t))
		if y1 is None:
			y1 = y1i.reshape((y1i.shape[0], 1, 1))
		else:
			y1 = np.concatenate((y1, 
                                             y1i.reshape((y1i.shape[0], 1, 1))), 
                                            axis=2)

	nd = int(np.round(dt/delta_t, 0))
	nf = int(np.round(tfinal/delta_t, 0))
	ndac = tp.shape[0]
	# This could be a way to check that we got a list of zpk/tf/lti objects 
	# instead of a single object. Right now, we do not check. 
	#if type(S) == tuple and type(S[0]) == tuple and np.isscalar(S[0][0]):
	#	S = [[lti(S)]]
	ni = len(S) # number of inputs
	
	if ni % ndac != 0:
		raise ValueError('The number of inputs must be divisible by the number of dac timings.')
		# Original comment from the MATLAB sources:
		# This requirement comes from the complex case, where the number of inputs
		# is 2 times the number of dac timings. I think this could be tidied up.

	nis = int(ni/ndac) # Number of inputs grouped together with a common DAC timing
	                   # (2 for the complex case)
	# Notice the number of outputs is hard-coded to one in the following
	# delsigma actually allows for supplying pulse() objects that have 
	# a variable number of outputs. As we have no MIMO description available,
	# we consider only 1-output systems
	if not nosum: # Sum the responses due to each input set
		y = np.zeros((np.ceil(tfinal/float(dt)) + 1, 1, nis))
	else:
		y = np.zeros((np.ceil(tfinal/float(dt)) + 1, 1, ni))

	for i in range(ndac):
		n1 = int(np.round(tp[i, 0]/delta_t, 0))
		n2 = int(np.round(tp[i, 1]/delta_t, 0))
		z1 = (n1, y1.shape[1], nis)
		z2 = (n2, y1.shape[1], nis)
		yy = + np.concatenate((np.zeros(z1), y1[:nf-n1+1, :, i*nis:(i + 1)*nis]), axis=0) \
		     - np.concatenate((np.zeros(z2), y1[:nf-n2+1, :, i*nis:(i + 1)*nis]), axis=0)
		yy = yy[::nd, :, :]
		if not nosum: # Sum the responses due to each input set
			y = y + yy
		else:
			y[:, :, i] = yy.reshape(yy.shape[0:2])
	return y

def test_pulse():
	"""Test function for pulse()"""
	import pkg_resources
	import scipy.io
	H = ([1], [1, 2, 10])
	pp = pulse([H], tp=(0., 1.), dt=.1, tfinal=10., nosum=False)
	pp = pp.reshape((pp.shape[0],1))
	fname = pkg_resources.resource_filename(__name__, "test_data/test_pulse.mat")
	pp2 = scipy.io.loadmat(fname)['pp']
	assert np.allclose(pp, pp2, atol=1e-6, rtol=1e-4)
	# FIXME ALSO CHECK MIMO TFS




# -*- coding: utf-8 -*-
# _DocumentNTF.py
# Module providing DocumentNTF
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the DocumentNTF function.
"""

from __future__ import division
import numpy as np
import pylab as plt

from ._plotPZ import plotPZ
from ._ds_freq import ds_freq
from ._dbv import dbv
from ._evalTF import evalTF
from ._ds_f1f2 import ds_f1f2
from ._rmsGain import rmsGain
from ._calculateTF import calculateTF
from ._infnorm import infnorm
from ._figureMagic import figureMagic

def DocumentNTF(arg1, osr=64, f0=0, quadrature=False):
	"""Plot the NTF's poles and zeros as well as its frequency-response

	The first argument is either the NTF or ABCD matrix. 
	If the first argument is ABCD, the STF is also plotted.

	.. plot::

	   from deltasigma import *
	   import pylab as plt
	   order = 4
	   osr = 64
	   nlev = 2
	   f0 = 0.
	   Hinf = 1.5
	   form = 'CRFB'
	   ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
	   a, g, b, c = realizeNTF(ntf, form)
	   b = np.hstack(( # Use a single feed-in for the input
	                  np.atleast_2d(b[0, 0]),
	                  np.zeros((1, max(b.shape)-1))
	                ))
	   ABCD = stuffABCD(a, g, b, c, form)
	   DocumentNTF(ABCD, osr, f0)

	"""
	if isinstance(arg1, np.ndarray):
		ABCD = arg1
		ntf, stf = calculateTF(ABCD)
	else:
		ntf = arg1
		stf = None
	
	fig = plt.figure(figsize=(12, 5))
	plt.subplot(121)
	plotPZ(ntf, 'b', 6, showlist=False)
	plt.title('Poles and Zeros')
	plt.subplot(122)
	f = ds_freq(osr, f0, quadrature)
	z = np.exp(2j * np.pi * f)
	H = dbv(evalTF(ntf, z))
	plt.plot(f, H, 'b')
	
	if stf is not None:
		fig.suptitle('NTF and STF', fontsize=14)
		G = dbv(evalTF(stf, z))
		plt.hold(True)
		plt.plot(f, G, 'm')
		plt.hold(False)
	else:
		fig.suptitle('NTF', fontsize=14)
	
	f1, f2 = ds_f1f2(osr, f0, quadrature)
	NG0 = dbv(rmsGain(ntf, f1, f2))
	plt.hold(True)
	plt.plot(np.array([f1, f2]), NG0*np.array([1, 1]), 'k', linewidth=3)
	
	if f0  ==  0:
		plt.text(0.5/osr, NG0, '  %.0fdB' % NG0, horizontalalignment = 'left', 
		         verticalalignment = 'center')
	else:
		plt.text(f0, NG0 + 1, '%.0fdB' % NG0, horizontalalignment = 'center', 
		         verticalalignment = 'bottom')
	msg = ' Inf-norm of H = %.2f\n 2-norm of H = %.2f' % (infnorm(ntf)[0], rmsGain(ntf, 0, 1))
	if quadrature:
		ING0 = dbv(rmsGain(ntf, - f1, - f2))
		plt.plot(-np.array([f1, f2]), ING0*np.array([1, 1]), 'k', linewidth=3)
		plt.text(-f0, ING0 + 1, '%.0fdB' % ING0, horizontalalignment = 'center', 
		         verticalalignment = 'bottom')
		f_left = - 0.5
	else:
		f_left = 0
	# variable 'f_left' used before assignment in DocumentNTF.m #REP
	if f0 < 0.25:
		plt.text(0.48, 0, msg, horizontalalignment = 'right', verticalalignment = 'top')
	else:
		plt.text(f_left, 0, msg, horizontalalignment = 'left', verticalalignment = 'top')
	plt.grid(True)
	y_bot = min(-80, np.round(NG0*1.1, -1))
	figureMagic(xRange=(f_left, 0.5), dx=1/20., yRange=(y_bot, 15), dy=10)
	plt.ylabel('|H(f)|')
	plt.xlabel('frequency')
	plt.title('Frequency Response')
	return 

def test_DocumentNTF():
	"""Test function for DocumentNTF"""
	from ._synthesizeNTF import synthesizeNTF
	from ._realizeNTF import realizeNTF
	from ._stuffABCD import stuffABCD
	order = 4
	osr = 64
	nlev = 2
	f0 = 0.
	Hinf = 1.5
	form = 'CRFB'
	ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
	a, g, b, c = realizeNTF(ntf, form)
	b = np.hstack(( # Use a single feed-in for the input
	               np.atleast_2d(b[0, 0]),
	               np.zeros((1, max(b.shape)-1))
	              ))
	ABCD = stuffABCD(a, g, b, c, form)
	DocumentNTF(ABCD, osr, f0)




# -*- coding: utf-8 -*-
# _figureMagic.py
# Module providing figureMagic()
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the utility function figureMagic()
"""

from __future__ import division
from warnings import warn
import numpy as np
import pylab as plt

def figureMagic(xRange=None, dx=None, xLab=None, yRange=None, dy=None, yLab=None, 
	        size=None, name=None):
	"""Utility function to quickly set plot parameters.

	**Parameters:**

	xRange : 2 elements sequence, optional
	    set the x-axis limits

	dx : scalar, optional
	    set the ticks spacing on the x-axis

	xLab : any, optional
	    Ignored variable, only accepted for compatibility with the MATLAB
	    syntax.

	yRange : 2 elements sequence, optional
	    set the y-axis limits

	dy : scalar, optional
	    set the ticks spacing on the y-axis

	yLab : any, optional
	    Ignored variable, only accepted for compatibility with the MATLAB
	    syntax.

	size : 2-elements sequence, optional
	    Figure size, in inches.

	name : string, optional
	    Title for the current plot (or subplot)

	All parameters are optional and any unspecified figure parameter is 
	left untouched.
	
	.. plot::

	   import numpy as np
	   import pylab as plt
	   from deltasigma import figureMagic
	   t = np.linspace(0, 1)
	   a = np.sin(2*np.pi*t + np.pi/4)
	   plt.subplot(121)
	   plt.plot(t, a)
	   plt.title("Before")
	   plt.subplot(122)
	   plt.plot(t, a)
	   plt.title("After")
	   figureMagic([0, 1], dx=.1, xLab=None, yRange=[-1.2, 1.2],
	               dy=.2, yLab=None, size=(8, 4), name="After")

	"""
	fig = plt.gcf()
	if size is not None and len(size):
		fig.set_size_inches(size)
	
	plt.grid(True)

	ax = plt.gca()
	if name is not None:
		ax.set_title(name, fontsize=14)

	if xRange is not None or yRange is not None:
		ax.set_autoscale_on(False)
		ax.set_aspect('auto', 'box')

	if xRange is not None:
		ax.set_xlim(xRange)
	if yRange is not None:
		ax.set_ylim(yRange)

	if dx is not None:
		x1, x2 = ax.get_xlim()
		ax.xaxis.set_ticks(np.arange(x1, x2, dx))
	if dy is not None:
		y1, y2 = ax.get_ylim()
		ax.yaxis.set_ticks(np.arange(y1, y2, dy))
	#ax.xaxis.set_major_formatter(ticker.FormatStrFormatter('%0.1f'))
	
	if xLab is not None:
		warn('figureMagic() got xLab=%s, but xLab is not implemented and will be ignored.' %
			 xLab)
	if yLab is not None:
		warn('figureMagic() got yLab=%s, but xLab is not implemented and will be ignored.' %
			 yLab)
	return

def test_figureMagic():
	"""Test function for figureMagic()"""
	a = np.arange(10)
	plt.figure()
	plt.plot(a)
	figureMagic(xRange=[1, 10], dx=1, xLab=None, yRange=[2, 8], dy=.5, yLab=None,
	            size=(10, 6), name="Test plot")



# -*- coding: utf-8 -*-
# _db.py
# This module provides the db function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the db() function, used to convert an RMS voltage,
expressed in Volt, or a power, expressed in Watt, to dBm.
"""

from __future__ import division
import numpy as np

from ._dbp import dbp
from ._dbv import dbv

def db(x, input_type='voltage', R=1.):
	"""The dB equivalent of the RMS signal ``x``.
	
	For input type *'voltage'*, the return value is defined as
	
	.. math::

	    P_{dB} = 20 \\mathrm{log}_{10}\\left(\\frac{x}{R}\\right)


	Otherwise, for input type *'power'*,

	.. math::

	    P_{dB} = 10 \\mathrm{log}_{10}(x)


	**Parameters:**

	x : scalar or sequence
	    The signal to be converted.

	input_type : string, optional
            The input type, either "voltage" or "power"

	R : float, optional
	    The normalization resistor value, used only for voltages. 


	**Returns:**

	PdB : scalar or sequence
	    The input expressed in dB.
	

	.. note:: MATLAB provides a function with this exact signature.

	.. seealso:: :func:`undbm`, :func:`undbv`, :func:`undbp`, :func:`dbv`, :func:`dbp`, :func:`dbv`

	"""
	if input_type.lower().strip() == 'voltage':
		y = dbv(x) - 10.*np.log10(R)
	elif input_type.lower().strip() == 'power':
		y = dbp(x)
		if R != 1.:
			warn("db called with a non default R value, " + 
			     "but R is going to be ignored since input_type is power",
				 RuntimeWarning)
	else:
		raise ValueError("db got input_type %s, instead of voltage or power" % input_type)
	return y

def test_db():
	"""Test function for db()
	"""
	from ._undbv import undbv
	tv = np.array([2])
	r = np.array([3.01029996])
	res = db(tv, 'power')
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
	tv = 2
	r = 3.01029996
	res = db(tv, 'power') 
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
	tv = 2, 2
	r = 3.01029996, 3.01029996
	res = db(tv, 'power')
	assert np.allclose(r, res, atol=1e-8, rtol=1e-5)
	t = np.array([3.0])
	r1 = undbv(db(t, 'voltage'))
	assert np.allclose(t, r1, atol=1e-8, rtol=1e-5)



# -*- coding: utf-8 -*-
# _synthesizeNTF0.py
# Module providing the synthesizeNTF function
# Copyright 2013 Giuseppe Venturini
# This file is distributed with python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# The following code has been (slightly) modified from pydsm, its original 
# copyright notice follows:
#
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The py dsm code was ported from the MATLAB Delta Sigma toolbox
# Copyright (c) 2009, Richard Schreier
# 
# The three software follow the same license, known as the 3-clause BSD. 
# See the LICENSE file for details.

"""
Synthesize a noise transfer function (NTF) for a delta-sigma modulator without
optimizing the result.
"""

import numpy as np
from warnings import warn
from ._evalTF import evalTF
from ._utils import cplxpair, zpk
from ._ds_optzeros import ds_optzeros
from ._config import itn_limit

def synthesizeNTF0(order, osr, opt, H_inf, f0):
	"""
	Synthesize a noise transfer function for a delta-sigma modulator.
	
	::warn This function is not meant to be used directly, instead, set optimize_NTF to False 
	and call synthesizeNTF(...).

	Parameters
	----------
	order : int, 
	    the order of the modulator
	osr : float, 
	    the oversamping ratio
	opt : int or list of floats
	    flag for optimized zeros

	    * 0 -> not optimized,
	    * 1 -> optimized,
	    * 2 -> optimized with at least one zero at band-center,
	    * 3 -> optimized zeros (with optimizer)
	    * 4 -> same as 3, but with at least one zero at band-center
	    * [z] -> zero locations in complex form

	H_inf : real
	    max allowed peak value of the NTF.
	f0 : real
	    center frequency for BP modulators, or 0 for LP modulators.
	    Note that 1 corresponds to the sampling frequency, so that 0.5 is the
	    maximum value. Value 0 specifies an LP modulator.

	Returns
	-------
	ntf : tuple
	    noise transfer function in zpk form.

	Raises
	------
	ValueError
	    'Cannot synthesize NTF zeros' if an empty list is supplied for the zeros.

	    'Order must be even for a bandpass modulator.' if the order is
	    incompatible with the modulator type.

	    'The opt vector must be of length xxx' if opt is used to explicitly
	    pass the NTF zeros and these are in the wrong number.

	Warns
	-----
	    'Unable to achieve specified H_inf ...' if the desired H_inf
	    cannot be achieved.

	    'Iteration limit exceeded' if the routine converges too slowly.

	Notes
	-----
	This is actually a wrapper function which calls the appropriate version
	of synthesizeNTF, based on the module control flag `optimize_NTF` which
	determines whether to use optimization tools.

	Parameter H_inf is used to enforce the Lee stability criterion.

	See also:
	   clans()   "Closed-loop analysis of noise-shaper." An alternative
	             method for selecting NTFs based on the 1-norm of the 
	             impulse response of the NTF

	   synthesizeChebyshevNTF()    Select a type-2 highpass Chebyshev NTF.
	             This function does a better job than synthesizeNTF if osr
	             or H_inf is low.
	"""

	# Determine the zeros.
	if f0 != 0:
		# Bandpass design-- halve the order temporarily.
		order = order/2
		dw = np.pi/(2*osr)
	else:
		dw = np.pi/osr

	if opt.ndim == 0:
		# opt is a number
		if opt == 0:
			z = np.zeros(order)
		else:
			z = dw*ds_optzeros(order, opt)
		if z.size == 0:
			raise ValueError('Cannot synthesize NTF zeros')
		if f0 != 0:
			# Bandpass design-- shift and replicate the zeros.
			order = order*2
			z = z + 2*np.pi*f0
			z = np.vstack((z, -z)).transpose().flatten()
		z = np.exp(1j*z)
	else:
		z = opt

	p = np.zeros(order)
	k = 1
	fprev = 0

	if f0 == 0:
		# Lowpass design
		HinfLimit = 2**order
		# !!! The limit is actually lower for opt=1 and low OSR
		if H_inf >= HinfLimit:
			warn('Unable to achieve specified H_inf.\n'
				'Setting all NTF poles to zero.')
			p = np.zeros(order)
		else:
			x = 0.3**(order-1)   # starting guess
			for itn in xrange(1, itn_limit + 1):
				me2 = -0.5*(x**(2./order))
				w = (2*np.arange(1, order + 1) + 1)*np.pi/order
				mb2 = 1 + me2*np.exp(1j*w)
				p = mb2 - np.sqrt(mb2**2 - 1)
				# Reflect poles to be inside the unit circle
				out = abs(p) > 1
				p[out] = 1./p[out]
				# The following is not exactly what delsig does.
				# We do not have an identical cplxpair
				p = cplxpair(p)
				f = np.real(evalTF(zpk(z, p, k), -1))-H_inf
				if itn == 1:
					delta_x = -f/100.
				else:
					delta_x = -f*delta_x/(f - fprev)
				xplus = x + delta_x
				if xplus > 0:
					x = xplus
				else:
					x = x*0.1
				fprev = f
				if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
					break
				if x > 1e6:
					warn('Unable to achieve specified Hinf.\n'
						 'Setting all NTF poles to zero.')
					p = np.zeros(order)
					break
				if itn == itn_limit:
					warn('Iteration limit exceeded.')
	else:
		# Bandpass design
		x = 0.3**(order/2-1)   # starting guess (not very good for f0~0)
		if f0 > 0.25:
			z_inf = 1.
		else:
			z_inf = -1.
		c2pif0 = np.cos(2*np.pi*f0)
		for itn in xrange(1, itn_limit+1):
			e2 = 0.5*x**(2./order)
			w = (2*np.arange(order)+1)*np.pi/order
			mb2 = c2pif0 + e2*np.exp(1j*w)
			p = mb2 - np.sqrt(mb2**2-1)
			# Reflect poles to be inside the unit circle
			out = abs(p)>1
			p[out] = 1/p[out]
			# The following is not exactly what delsig does.
			p = cplxpair(p)
			f = np.real(evalTF((z, p, k), z_inf)) - H_inf
			if itn == 1:
				delta_x = -f/100
			else:
				delta_x = -f*delta_x/(f - fprev)
			xplus = x + delta_x
			if xplus > 0:
				x = xplus
			else:
				x = x*0.1
			fprev = f
			if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
				break
			if x > 1e6:
				warn('Unable to achieve specified Hinf.\n'
					'Setting all NTF poles to zero.')
				p = np.zeros(order)
				break
			if itn == itn_limit:
				warn('Iteration limit exceeded.')

	z = cplxpair(z)
	return z, p, k



# -*- coding: utf-8 -*-
# _simulateSNR.py
# Module providing the simulateSNR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the simulateSNR() function
"""

from __future__ import division
from warnings import warn
import numpy as np
from numpy.fft import fft, fftshift

from ._simulateDSM import simulateDSM
from ._calculateSNR import calculateSNR
from ._mapQtoR import mapQtoR

def simulateSNR(arg1, osr, amp=None, f0=0, nlev=2, f=None, k=13,
                quadrature=False):
    """Determine the SNR for a delta-sigma modulator by using simulations.

    Simulate a delta-sigma modulator with sine wave inputs of various
    amplitudes and calculate the signal-to-noise ratio (SNR) in dB for each 
    input.

    Two alternative descriptions of the modulator can be used:

     * The modulator is described by a noise transfer function (NTF), provided
       as ``arg1`` and the number of quantizer levels (``nlev``).

     * Alternatively, the first argument to simulateSNR may be an ABCD matrix or
       the name of a function taking the input signal as its sole argument.

    The band of interest is defined by the oversampling ratio (``osr``)
    and the center frequency (``f0``).

    The input signal is characterized by the ``amp`` vector and the ``f`` variable.
    A default value for ``amp`` is used if not supplied.

    ``f`` is the input frequency, normalized such that 1 -> fs;
    ``f`` is rounded to an FFT bin.

    Using sine waves located in FFT bins, the SNR is calculated as the ratio
    of the sine wave power to the power in all in-band bins other than those
    associated with the input tone. Due to spectral smearing, the input tone
    is not allowed to lie in bins 0 or 1. The length of the FFT is :math:`2^k`.

    If ntf is complex, simulateQDSM (which is slow) is called.
    If ABCD is complex, simulateDSM is used with the real equivalent of ABCD
    in order to speed up simulations.

    Future versions may accommodate STFs.

    **Parameters:**

    arg1 : scipy 'lti' object, or ndarray
        The first argument may be one of the various supported representations
        for a (SISO) transfer function or an ABCD matrix.

    osr : int
        The over-sampling ratio.

    amp : sequence, optional
        The amplitudes in dB, referred to the FS, for which the SNR is to be 
        evaluated. ``amp`` defaults to [-120 -110...-20 -15 -10 -9 -8 ... 0]dB,
        where 0 dB means a full-scale (peak value = nlev-1) sine wave.

    f0 : float, optional
        The center frequency. Normalized. Defaults to 0.

    nlev : int, optional
        Number of quantizer levels, defaults to 2.

    f : float, optional
        Test signal input frequency. Normalized. Rounded to an FFT bin.
        Defaults to:

    .. math::

        f = \\frac{1}{4\ \mathrm{OSR}}

    k : int, optional
        The number of samples used to compute the FFT is set by the integer `k`
        - default value 13 - through:

    .. math::

        N_{samples} = 2^k

    quadrature : boolean, optional
        Whether the delta sigma modulator is a quadrature modulator or not.
        Defaults to ``False``.

    .. note:: Setting ``quadrature`` to ``True`` results in a \\
        ``NotImplementedError`` being raised, as :func:`simulateQDSM` has not been \\
        implemented yet.

    **Returns:**

    snr : ndarray
        The SNR, from simulation

    amp : ndarray
        The amplitudes corresponding to the SNR values.

    **Example:**

    Compare the SNR vs input amplitude curve for a fifth-order modulator, as 
    determined by the describing function method (:func:`predictSNR`) with
    that determined by simulation (:func:`simulateSNR`).::

        import pylab as plt
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        snr_pred, amp, _, _, _ = predictSNR(H,OSR)
        snr, amp = simulateSNR(H, OSR)
        plt.plot(amp, snr_pred, 'b', amp, snr, 'go')
        plt.grid(True)
        figureMagic([-100, 0], 10, None,
                    [0, 100], 10, None)
        plt.xlabel('Input Level, dB')
        plt.ylabel('SNR dB')
        s = 'peak SNR = %4.1fdB\\n' % max(snr)
        plt.text(-65, 15, s, horizontalalignment='left')

    .. plot::

        import pylab as plt
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        snr_pred, amp, _, _, _ = predictSNR(H,OSR)
        snr, amp = simulateSNR(H, OSR)
        plt.plot(amp, snr_pred, 'b', amp, snr, 'go')
        plt.grid(True)
        figureMagic([-100, 0], 10, None,
                    [0, 100], 10, None)
        plt.xlabel('Input Level, dB')
        plt.ylabel('SNR dB')
        s = 'peak SNR = %4.1fdB\\n' % max(snr)
        plt.text(-65, 15, s, horizontalalignment='left')

    """
    # Look at arg1 and decide if the system is quadrature
    quadrature_ntf = False
    if callable(arg1):
        pass
    elif hasattr(arg1, '__class__') and arg1.__class__.__name__ == 'lti':
        # FIXME lti/tf object
        pz = np.vstack((arg1.p[0], arg1.z[0]))
        for i in range(1, 3):
            if np.any(np.abs(np.imag(np.poly(pz[:, i-1]))) > 0.0001):
                quadrature = True
                quadrature_ntf = True
    else: # ABCD matrix
        if not np.all(np.imag(arg1) == 0):
            quadrature = True

    if amp is None:
        amp = np.concatenate((
                              np.arange(- 120, -20 + 1, 10),
                              np.array((-15,)),
                              np.arange(-10, 1)
                            ))
    elif not hasattr(amp, '__len__'):
        amp = np.array((amp, ))
    else:
        amp = np.asarray(amp)
    osr_mult = 2
    if f0 != 0 and not quadrature:
        osr_mult = 2*osr_mult
    if f is None or np.isnan(f):
        f = f0 + 0.5/(osr*osr_mult) # Halfway across the band
    M = nlev - 1
    if quadrature and not quadrature_ntf:
        # Modify arg1 (ABCD) and nlev so that simulateDSM can be used
        nlev = np.array([nlev, nlev]).reshape(1, -1)
        arg1 = mapQtoR(arg1)
    if abs(f - f0) > 1/(osr*osr_mult):
        warn('The input tone is out-of-band.')
    N = 2**k
    if N < 8*2*osr:
        warn('Increasing k to accommodate a large oversampling ratio.')
        k = np.array(np.ceil(np.log2(8*2*osr)), dtype=np.int64)
        N = 2**k
    F = np.round(f*N)
    if np.abs(F) <= 1:
        warn('Increasing k to accommodate a low input frequency.')
        # Want f*N >= 1
        k = np.ceil(np.log2(1./f))
        N = 2**k
        F = 2
    Ntransient = 100
    soft_start = 0.5*(1 - np.cos(2*np.pi/Ntransient * \
                                 np.arange(0, Ntransient/2)))
    if not quadrature:
        tone = M*np.sin(2*np.pi*F/N*np.arange(0, N + Ntransient))
        tone[:Ntransient/2] = tone[:Ntransient/2] * soft_start
    else:
        tone = M * np.exp(2j*np.pi*F/N * np.arange(0, N + Ntransient))
        tone[:Ntransient/2] = tone[:Ntransient/2] * soft_start
        if not quadrature_ntf:
            tone = np.hstack((np.real(tone), np.imag(tone)))
    # create a Hann window
    window = 0.5*(1 - np.cos(2*np.pi*np.arange(0, N)/N))
    if quadrature:
        window = np.vstack((window, window))
    if f0 == 0:
        inBandBins = int(N/2) + np.arange(3,
                                     np.round(N/osr_mult/osr) + 1,
                                     dtype=np.int32)
        F = F - 2
    else:
        f1 = np.round(N*(f0 - 1./osr_mult/osr))
        # Should exclude DC
        inBandBins = int(N/2) + np.arange(f1,
                                     np.round(N*(f0 + 1./osr_mult/osr)) + 1,
                                     dtype=np.int32)
        F = F - f1 + 1
    snr = np.zeros(amp.shape)
    i = 0
    for A in np.power(10.0, amp/20):
        if callable(arg1):
            v = arg1(A*tone)
        else:
            if quadrature_ntf:
                raise NotImplementedError("simulateQDSM has not been \
                                           implemented yet.")
                #v = simulateQDSM(A*tone, arg1, nlev)
            else:
                v, _, _, _ = simulateDSM(A*tone, arg1, nlev)
                if quadrature:
                    v = v[0, :] + 1j*v[1, :]
        #print window.shape, type(v[Ntransient:N + Ntransient])
        hwfft = fftshift(fft(window*v[Ntransient:N + Ntransient]))
        snr[i] = calculateSNR(hwfft[inBandBins - 1], F)
        i += 1
    return snr, amp



# -*- coding: utf-8 -*-
# _realizeNTF.py
# Module providing realizeNTF
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing realizeNTF()
"""

from __future__ import division, print_function
from warnings import warn

import numpy as np
from scipy.signal import tf2zpk

from ._calculateTF import calculateTF
from ._evalRPoly import evalRPoly
from ._evalTF import evalTF
from ._stuffABCD import stuffABCD
from ._utils import carray, cplxpair, _get_zpk

def realizeNTF(ntf, form='CRFB', stf=None):
	"""Convert an NTF into coefficients for the desired structure.

	**Parameters:**

	ntf : a zpk transfer function
	    The modulator noise transfer function (NTF)

	form : string
	    A structure identifier.

	Supported structures are:
	
	* *"CRFB"*: Cascade of resonators, feedback form.

 	* *"CRFF"*: Cascade of resonators,  feedforward form.

	* *"CIFB"*: Cascade of integrators,  feedback form. 

	* *"CIFF"*: Cascade of integrators,  feedforward form.

 	* *"CRFBD"*: CRFB with delaying quantizer.

 	* *"PFF"*: Parallel feed-forward.

	* *"Stratos"*: A CIFF-like structure with non-delaying resonator feedbacks, contributed to the MATLAB delta sigma toolbox in 2007 by Jeff Gealow

	See the accompanying documentation (:ref:`topologies-diagrams`) for
	block diagrams of each structure.

	.. note:
	The order of the NTF zeros must be (real, complex conj. pairs).
	The order of the zeros is used when mapping the NTF onto the chosen topology.

	stf : a zpk transfer function, optional
	    the Signal Transfer Function
	
	**Returns:**

	a, g, b, c : tuple of ndarrays
	    the coefficients for the desired structure

	**Example:**

	Determine the coefficients for a 5th-order modulator with the
	cascade-of-resonators structure, feedback (CRFB) form.::

	    from deltasigma import synthesizeNTF, realizeNTF
	    H = synthesizeNTF(5, 32, 1)
	    a, g, b, c = realizeNTF(H,'CRFB')

	Returns the values::

	    a: 0.0007, 0.0084, 0.055, 0.2443, 0.5579
	    g: 0.0028, 0.0079
	    b: 0.0007, 0.0084, 0.055, 0.2443, 0.5579, 1.0
	    c: 1.0, 1.0, 1.0, 1.0, 1.0

	"""

	# The basic idea is to equate the loop filter at a set of
	# points in the z-plane to L1  =  1-1/ntf at those points.

	# Code common to all functions\
	if (hasattr(ntf, 'inputs') and not ntf.inputs == 1) or \
	   (hasattr(ntf, 'outputs') and not ntf.outputs == 1):
		raise TypeError("Only SISO transfer functions can be evaluated.")

	ntf_z, ntf_p, _k = _get_zpk(ntf)
	ntf_z = carray(ntf_z)
	ntf_p = carray(ntf_p)
	order = max(ntf_p.shape)
	order2 = int(np.floor(order/2))
	odd = (order - 2*order2 == 1)
	
	a = np.zeros((1, order))
	g = np.zeros((1, order2))
	b = np.zeros((1, order + 1))
	c = np.ones((1, order))
	T = np.zeros((order, order*2), dtype='complex')

	# instead of asuming enforce roots order
	ntf_z = cplxpair(ntf_z)[::-1]

	N = 200
	min_distance = 0.09
	zSet = np.zeros((N,), dtype='complex')
	j = 0
	for i in range(1, N + 1):
		z = 1.1 * np.exp(2j*np.pi*i/N)
		if np.all(np.abs(ntf_z - z) > min_distance):
			zSet[j] = z
			j = j + 1
	zSet = zSet[:j]
	if form == 'CRFB':
		# Find g
		# Assume the roots are ordered, real first, then cx conj. pairs 
		for i in range(0, order2):
			g[0, i] = 2*(1 - np.real(ntf_z[2*i + 1*odd]))
		L1 = np.zeros((1, order*2), dtype='complex')
		# Form the linear matrix equation a*T* = L1
		for i in range(0, 2*order):
			z = zSet[i]
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			Dfactor = (z - 1)/z
			product = 1
			for j in range(order, 0 + 1*odd, -2):
				product = z/evalRPoly(ntf_z[j - 2:j], z)*product
				T[j - 1, i] = product * Dfactor
				T[j - 2, i] = product
			if odd:
				T[0, i] = product/(z - 1)
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b[0, :order] = a[0, :]
			b[0, order] = 1
	elif form == 'CRFF':
		# Find g
		# Assume the roots are ordered, real first, then cx conj. pairs
		for i in range(0, order2):
			g[0, i] = 2*(1 - np.real(ntf_z[2*i + 1*odd]))
		L1 = np.zeros((1, order*2), dtype='complex')
		# Form the linear matrix equation a*T*=L1
		for i in range(0, 2*order):
			z = zSet[i]
			# L1(z) = 1-1/H(z)
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			if odd:
				Dfactor = z - 1
				product = 1/Dfactor
				T[0, i] = product
			else:
				Dfactor = (z - 1)/z
				product = 1.
			for j in range(0 + 1*odd, order, 2):
				product = z/evalRPoly(ntf_z[j:j + 2], z)*product
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b = np.hstack((np.atleast_2d(1), 
			               np.zeros((1, order - 1)), 
			               np.atleast_2d(1)
			             ))
	elif form == 'CIFB':
		if any(abs(np.real(ntf_z) - 1) > 0.001):
			warn("The ntf's zeros have had their real parts set to one.")
		ntf_z = 1 + 1j*np.imag(ntf_z)
		for i in range(order2):
			g[0, i] = np.imag(ntf_z[2*i + 1*odd])**2
		L1 = np.zeros((1, order*2), dtype='complex')
		for i in range(order*2):
			z = zSet[i]
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			Dfactor = z - 1.
			product = 1.
			for j in range(order - 1, 1*odd, -2):
				product = product/evalRPoly(ntf_z[j-1:j+1], z)
				T[j, i] = product * Dfactor
				T[j - 1, i] = product
			if odd:
				T[0, i] = product/(z - 1.)
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b[0, :order] = a
			b[0, order] = 1.
	elif form == 'CIFF':
		if (abs(np.real(ntf_z) - 1) > 0.001).any():
			warn("The ntf's zeros have had their real parts set to one.")
		ntf_z = 1. + 1j*np.imag(ntf_z)
		for i in range(order2):
			g[0, i] = np.imag(ntf_z[2*i + 1*odd])**2
		L1 = np.zeros((1, order*2), dtype='complex')
		for i in range(order*2):
			z = zSet[i]
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			Dfactor = (z - 1)
			if odd:
				product = 1./(z - 1.)
				T[0, i] = product
			else:
				product = 1
			for j in range(0 + 1*odd, order - 1, 2):
				product = product/evalRPoly(ntf_z[j:j + 2], z)
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b = np.zeros((1, order+1))
			b[0, 0] = 1.
			b[0, -1] = 1.
	elif form == 'CRFBD':
		for i in range(order2):
			g[0, i] = 2*(1. - np.real(ntf_z[2*i + 1*odd]))
		L1 = np.zeros((1, order*2), dtype='complex')
		for i in range(order*2):
			z = zSet[i]
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			Dfactor = (z - 1.)
			product = 1./z
			for j in range(order - 1, 0 + 1*odd, -2):
				product = z/evalRPoly(ntf_z[j - 1:j + 1], z) * product
				T[j, i] = product * Dfactor
				T[j - 1, i] = product
			if (odd):
				T[0, i] = product*z/(z - 1.)
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b = np.hstack((a, np.zeros((1, 1))))
			b[0, order] = 1.
	elif form == 'CRFFD':
		raise ValueError('I am sorry to inform you, the CRFFD is buggy in' +
		                 ' MATLAB and its Python port is equally buggy.\n' +
		                 'To this day, it is unclear what the original' +
		                 ' author meant with this code.\n'
		                 'If you know how to fix this, feel free to mail' +
		                 'me at ggventurini+GITHUB@gmail.com')
		# Find g
    		# Assume the roots are ordered, real first, then cx conj. pairs
		for i in range(order2):
			g[0, i] = 2*(1. -np.real(ntf_z[2*i + 1*odd]))
		# zL1 = z*(1-1/H(z))
		zL1 = np.atleast_2d(zSet * (1. - 1.0/evalTF(ntf, zSet)))
		# Form the linear matrix equation a*T*=zL1
		for i in range(order*2):
			z = zSet[i]
			if odd:
				Dfactor = (z - 1.)/z
				product = 1./Dfactor
				T[0, i] = product
			else:
				Dfactor = z - 1
				product = 1
			for j in range(0 + 1*odd, order, 2):
				product = z/evalRPoly(ntf_z[j:j + 2], z)*product
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
		a = -np.real(np.linalg.lstsq(T.T, zL1.T)[0]).T
		if stf is None:
			b = np.hstack((np.atleast_2d(1), 
			               np.zeros(1, order - 1), 
			               np.atleast_2d(1))
			             )
	elif form == 'PFF':
		warn("Untested code accessed: realizeNTF('PFF')")
		# Find g
		# Assume the roots are ordered, real first, then cx conj. pairs
		# with the secondary zeros after the primary zeros
		for i in range(order2):
			g[0, i] = 2*(1. - np.real(ntf_z[2*i + 1*odd]))
		# Find the dividing line between the zeros
		theta0 = np.abs(np.angle(ntf_z[0]))
		# !! 0.5 radians is an arbitrary separation !!
		i = np.flatnonzero(np.abs(np.abs(np.angle(ntf_z)) - theta0) > 0.5)
		if i.shape[0]:
			order_1 = i[0] - 1
		else:
			order_1 = 0
		order_2 = order - order_1
		if i.shape[0] !=  order_2:
			raise ValueError('For the PFF form, the NTF zeros must ' +
			                 'be sorted into primary and secondary zeros')
		odd_1 = order_1 % 2
		odd_2 = order_2 % 2
		L1 = np.zeros((1, order*2), dtype='complex')
		# Form the linear matrix equation a*T*=L1
		for i in range(order*2):
			z = zSet[i]
			# L1(z) = 1-1/H(z)
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			if odd_1:
				Dfactor = z - 1
				product = 1./Dfactor
				T[0, i] = product
			else:
				Dfactor = (z - 1)/z
				product = 1
			for j in range(odd_1, order_1, 2):
				product = z/evalRPoly(ntf_z[j:j + 2], z) * product
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
			if odd_2:
				Dfactor = z - 1
				product = 1./Dfactor
				T[order_1, i] = product
			else:
				Dfactor = (z - 1)/z
				product = 1.
			for j in range(order_1 + odd_2, order, 2):
				product = z/evalRPoly(ntf_z[j:j + 2], z)*product
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
		a = -np.real(np.linalg.lstsq(T.T, zL1.T)[0]).T
		if stf is None:
			b = np.hstack((np.atleast_2d(1), 
			               np.zeros((1, order_1 - 1)), 
			               np.atleast_2d(1), 
			               np.zeros(1, order_2 - 1), 
			               np.atleast_2d(1)
			             ))
	elif form == 'Stratos':
		# code copied from case 'CRFF':
		# Find g
		# Assume the roots are ordered, real first, then cx conj. pairs
		for i in range(order2):
			g[0, i] = 2*(1 - np.real(ntf_z[2*i + 1*odd]))
		# code copied from case 'CIFF':
		L1 = np.zeros((1, order*2), dtype='complex')
		# Form the linear matrix equation a*T*=L1
		for i in range(order*2):
			z = zSet[i]
			# L1(z) = 1-1/H(z)
			L1[0, i] = 1. - evalRPoly(ntf_p, z)/evalRPoly(ntf_z, z)
			Dfactor = (z - 1.)
			if odd:
				product = 1./(z - 1.)
				T[0, i] = product
			else:
				product = 1.
			for j in range(0 + 1*odd, order - 1, 2):
				product = product/evalRPoly(ntf_z[j:j + 2], z)
				T[j, i] = product * Dfactor
				T[j + 1, i] = product
		a = -np.real(np.linalg.lstsq(T.T, L1.T)[0]).T
		if stf is None:
			b = np.hstack((
			               np.atleast_2d(1),
			               np.zeros((1, order - 1)), 
			               np.atleast_2d(1)
			             ))
	if not stf is None:
		# Compute the TF from each feed-in to the output 
		# and solve for coefficients which yield the best match
		# THIS CODE IS NOT OPTIMAL,  in terms of computational efficiency.
		stfList = []
		for i in range(0, order + 1):
			bi = np.zeros(1, order + 1)
			bi[i] = 1
			ABCD = stuffABCD(a, g, bi, c, form)
			if form[2:4] == 'FF':
				ABCD[0, order + 1] = - 1
			_, stfListi = calculateTF(ABCD)
			stfList.append(stfListi)
		# Build the matrix equation b A  =  x and solve it.
		A = np.zeros((order + 1, max(zSet.shape)))
		for i in range(0, order + 1):
			A[i, :] = evalTF(stfList[i], zSet)
		x = evalTF(stf, zSet)
		x = x[:].T
		b = np.real(x/A)
	return a, g, b, c
	
def test_realizeNTF():
	"""Test function for realizeNTF()"""
	from ._synthesizeNTF import synthesizeNTF
	orders = (2, 3, 4, 5)
	osr = 32
	nlev = 2
	f0s = (0., 0.25)
	Hinf = 1.5
	forms = ('CRFB', 'CRFF', 'CIFB', 'CIFF', 'CRFBD', 'Stratos')
	res = {0.:{'CIFB':{2:{'a':(0.2164, 0.7749),
	                      'g':(0, ),
	                      'b':(0.2164, 0.7749, 1.0000),
	                      'c':(1., 1. )
	                     },
	                   3:{'a':(0.0444, 0.2843, 0.8025),
	                      'g':(0.0058, ),
	                      'b':(0.0444, 0.2843, 0.8025, 1.),
	                      'c':(1., 1., 1.)
	                     },
	                   4:{'a':(0.0062, 0.0655, 0.3042, 0.8089),
	                      'g':(0., 0.0069),
	                      'b':(0.0062, 0.0655, 0.3042, 0.8089, 1.),
	                      'c':(1., 1., 1., 1)
	                     },
	                   5:{'a':(0.0007, 0.0095, 0.0731, 0.3100, 0.81309),
	                      'g':(0.0028, 0.0079),
	                      'b':(0.0007, 0.0095, 0.0731, 0.3100, 0.8130, 1.),
	                      'c':(1., 1., 1., 1., 1.)
	                     }
	                  },
	           'CRFB':{2:{'a':(0.2164, 0.5585),
	                      'g':(0, ),
	                      'b':(0.2164, 0.5585, 1.0000),
	                      'c':(1., 1. )
	                     },
	                   3:{'a':(0.0444, 0.2399, 0.5569),
	                      'g':(0.0058, ),
	                      'b':(0.0444, 0.2399, 0.5569, 1.),
	                      'c':(1., 1., 1.)
	                     },
	                   4:{'a':(0.0062, 0.0530, 0.2449, 0.5571),
	                      'g':(0, 0.0069),
	                      'b':(0.0062, 0.0530, 0.2449, 0.5571, 1.),
	                      'c':(1., 1., 1., 1)
	                     },
	                   5:{'a':(0.0007, 0.0084, 0.0550, 0.2443, 0.5579),
	                      'g':(0.0028, 0.0079),
	                      'b':(0.0007, 0.0084, 0.0550, 0.2443, 0.5579, 1.),
	                      'c':(1., 1., 1., 1., 1.)
	                     }
	                   },
	           'CRFF':{2:{'a':(0.5585, 0.2164),
	                      'g':(0, ),
	                      'b':(1., 0., 1.),
	                      'c':(1., 1. )
	                     },
	                   3:{'a':(0.5569, 0.2399, 0.0412),
	                      'g':(0.0058, ),
	                      'b':(1., 0., 0., 1.),
	                      'c':(1., 1., 1.)
	                     },
	                   4:{'a':(0.5571, 0.2449, 0.0492, 0.0046),
	                      'g':(0, 0.0069),
	                      'b':(1., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1)
	                     },
	                   5:{'a':(0.5579, 0.2443, 0.0505, 0.0071, 0.0003),
	                      'g':(0.0028, 0.0079),
	                      'b':(1., 0., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1., 1.)
	                     }
	                   },
	           'CIFF':{2:{'a':(0.7749, 0.2164),
	                      'g':(0., ),
	                      'b':(1., 0., 1.),
	                      'c':(1., 1. )
	                     },
	                   3:{'a':(0.8025, 0.2843, 0.0398),
	                      'g':(0.0058, ),
	                      'b':(1., 0., 0., 1.),
	                      'c':(1., 1., 1.)
	                     },
	                   4:{'a':(0.8089, 0.3042, 0.0599, 0.0041),
	                      'g':(0., 0.0069),
	                      'b':(1., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1)
	                     },
	                   5:{'a':(0.8130, 0.3100, 0.0667, 0.0080, 0.0001),
	                      'g':(0.0028, 0.0079),
	                      'b':(1., 0., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1., 1.)
	                     }
	                  },
	           'CRFBD':{2:{'a':(0.2164, 0.7749),
	                       'g':(0, ),
	                       'b':(0.2164, 0.7749, 1.),
	                       'c':(1., 1. )
	                      },
	                    3:{'a':(0.0444, 0.2399, 0.7967),
	                       'g':(0.0058, ),
	                       'b':(0.0444, 0.2399, 0.7967, 1.),
	                       'c':(1., 1., 1.)
	                      },
	                    4:{'a':(0.0062, 0.0592, 0.2449, 0.8020),
	                       'g':(0, 0.0069),
	                       'b':(0.0062, 0.0592, 0.2449, 0.8020, 1.),
	                       'c':(1., 1., 1., 1)
	                      },
	                    5:{'a':(0.0007, 0.0084, 0.0633, 0.2443, 0.8023),
	                       'g':(0.0028, 0.0079),
	                       'b':(0.0007, 0.0084, 0.0633, 0.2443, 0.8023, 1.),
	                       'c':(1., 1., 1., 1., 1.)
	                      }
	                   },
	           #'CRFFD':{2:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           },
	           #         3:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           },
	           #         4:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           },
	           #         5:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           }
	           #        }
	           'Stratos':{2:{'a':(0.7749, 0.2164),
	                       'g':(0, ),
	                       'b':(1, 0., 1.),
	                       'c':(1., 1. )
	                      },
	                    3:{'a':(0.7967, 0.2796, 0.0398),
	                       'g':(0.0058, ),
	                       'b':(1., 0., 0., 1.),
	                       'c':(1., 1., 1.)
	                      },
	                    4:{'a':(0.8020, 0.2987, 0.0579, 0.0042),
	                       'g':(0, 0.0069),
	                       'b':(1., 0., 0., 0., 1.),
	                       'c':(1., 1., 1., 1)
	                      },
	                    5:{'a':(0.8023, 0.3013, 0.0643, 0.0075, 0.0001),
	                       'g':(0.0028, 0.0079),
	                       'b':(1., 0., 0., 0., 0., 1.),
	                       'c':(1., 1., 1., 1., 1.)
	                      }
	                   }
	          },
	      0.25:{'CIFB':{2:{'a':(0.3333, 2.0000),
	                      'g':(1., ),
	                      'b':(0.3333, 2., 1.0000),
	                      'c':(1., 1. )
	                     },
	                   4:{'a':(-3.5585, 2.4503, 5.22512, 4.0000),
	                      'g':(1., 1.),
	                      'b':(-3.5585, 2.4503, 5.22512, 4.0000, 1.),
	                      'c':(1., 1., 1., 1)
	                     }
	                  },
	           'CRFB':{2:{'a':(-0.6667, 0.6667),
	                      'g':(2.0, ),
	                      'b':(-0.6667, 0.6667, 1.0000),
	                      'c':(1., 1. )
	                     },
	                   4:{'a':(-0.2164, 0., -0.5585, 0.5585),
	                      'g':(2.0, 2.0),
	                      'b':(-0.2164, 0.0, -0.5585, 0.5585, 1.),
	                      'c':(1., 1., 1., 1)
	                     }
	                   },
	           'CRFF':{2:{'a':(0.6667, -0.6667),
	                      'g':(2.0, ),
	                      'b':(1., 0., 1.),
	                      'c':(1., 1. )
	                     },
	                   4:{'a':(0.5585, -0.5585, 0., -0.2164),
	                      'g':(2., 2.),
	                      'b':(1., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1)
	                     }
	                   },
	           'CIFF':{2:{'a':(2., 0.3333),
	                      'g':(1., ),
	                      'b':(1., 0., 1.),
	                      'c':(1., 1. )
	                     },
	                   4:{'a':(4., 5.2251, 2.4503, -3.5585),
	                      'g':(1., 1.),
	                      'b':(1., 0., 0., 0., 1.),
	                      'c':(1., 1., 1., 1)
	                     }
	                  },
	           'CRFBD':{2:{'a':(-0.6667, 0.),
	                       'g':(2.0, ),
	                       'b':(-0.6667, 0., 1.),
	                       'c':(1., 1. )
	                      },
	                    4:{'a':(-0.2164, -0.2164, -0.5585, 0.),
	                       'g':(2., 2.),
	                       'b':(-0.2164, -0.2164, -0.5585, 0., 1.),
	                       'c':(1., 1., 1., 1)
	                      }
	                   },
	           #'CRFFD':{2:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           },
	           #         4:{'a':(),
	           #            'g':(),
	           #            'b':(),
	           #            'c':()
	           #           }
	           #        }
	           'Stratos':{2:{'a':(0., -0.6667),
	                       'g':(2., ),
	                       'b':(1, 0., 1.),
	                       'c':(1., 1. )
	                      },
	                    4:{'a':(0., -0.7749, 0., 0.2164),
	                       'g':(2.0, 2.0),
	                       'b':(1., 0., 0., 0., 1.),
	                       'c':(1., 1., 1., 1)
	                      }
	                   }
	          }
	      }

	for f0 in f0s:
		for form in forms:
			for order in orders:
				if f0 != 0. and order % 2 == 1:
					# odd-order pass band modulator
					continue
				# Optimized zero placement
				print("Testing form: %s, order: %d, f0: %f" % \
				      (form, order, f0))
				ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
				a, g, b, c = realizeNTF(ntf, form)
				print(a, g, b, c)
				assert np.allclose(a, res[f0][form][order]['a'], 
				            atol=1e-4, rtol=1e-3)
				assert np.allclose(g, res[f0][form][order]['g'], 
				            atol=1e-4, rtol=1e-3)
				assert np.allclose(b, res[f0][form][order]['b'], 
				            atol=1e-4, rtol=1e-3)
				assert np.allclose(c, res[f0][form][order]['c'], 
				            atol=1e-4, rtol=1e-3)
	return 



# -*- coding: utf-8 -*-
# _ds_quantize.py
# This module provides the ds_quantize function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_quantize() function, used to quantize signals
according to a user-specified quantizer characteristic.
"""

import numpy as np

def ds_quantize(y, n=2):
	"""Quantize ``y``

	Quantize y to: 

	 * an odd integer in [-n+1, n-1], if n is even, or
	 * an even integer in [-n+1, n-1], if n is odd.

	This definition gives the same step height for both mid-rise
	and mid-tread quantizers.
	``n`` can be a column vector which specifies how to quantize the 
	rows of ``y``.
	"""
	assert (np.round(n, 0) == n).all() # did we get an int or an array of int?
	if not (hasattr(n, 'shape')): 
		n = n*np.ones(y.shape) # we got an int
	else:
		assert len(n.shape) == 1 or 1 in n.shape
		n = (np.ones((max(n.shape), y.shape[1])).T*n).T

	i = (n % 2 == 0)	
	v = np.zeros(y.shape)
	v[i] = 2*np.floor(0.5*y[i]) + 1 	# mid-rise quantizer
	v[~i] = 2*np.floor(0.5*(y[~i] + 1))	# mid-tread quantizer

	# Limit the output
	L = n - 1
	for m in (-1, 1):
		i = (m*v > L)
		if i.any():
			v[i] = m*L[i]
	return v

def test_ds_quantize():
	"""Test function for ds_quantize()
	"""
	t = np.arange(-3, 3, .2)
	t = t.reshape((1, t.shape[0]))
	y = t
	for _ in range(2):
		y = np.concatenate((y, t), axis=0)
	n = np.array([2, 3, 4])
	re1 = ds_quantize(y, n)
	re2 = np.array([[-1., -1., -1., -1., -1., -1., -1., -1., -1., -1., -1., -1., -1.,
	                 -1., -1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,
	                  1.,  1.,  1.,  1.],
	                [-2., -2., -2., -2., -2., -2., -2., -2., -2., -2.,  0.,  0.,  0.,
	                  0.,  0.,  0.,  0.,  0.,  0.,  0.,  2.,  2.,  2.,  2.,  2.,  2.,
	                  2.,  2.,  2.,  2.],
	                [-3., -3., -3., -3., -3., -1., -1., -1., -1., -1., -1., -1., -1.,
	                 -1., -1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  3.,
	                  3.,  3.,  3.,  3.]
	               ])
	np.allclose(re1, re2, atol=1e-8, rtol=1e-5)




# -*- coding: utf-8 -*-
# _lollipop.py
# The lollipop function: plot digital samples like it was a lollipop field.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Lollipop plotting function.
"""

import numpy as np
import pylab as plt
from numpy.matlib import repmat

# Plot lollipops (o's and sticks)
# 
#	   ^
#	   | o     o     o         o o o   o o 
#	   | |   o | o o | o     o | | | o | | 
#	   | | o | | | | | | o o | | | | | | | 
#          +----------------------------------->

def lollipop(x, y, color='b', lw=2, ybot=0):
	"""Plot lollipops (o's and sticks)
	
	**Parameters:**

	x, y : ndarray
	       data to be plotted

	color : any matplotlib color
	        plotting color

	lw : float
             line width value in points

	ybot : float
               ground level for the sticks.

	**Returns:**

	None
	
	**Example:**

	.. plot::

	    import matplotlib.pyplot as plt
	    import numpy as np
	    from deltasigma import lollipop
	    t = np.arange(1, 20)*1e-3
	    f = 20.
	    a = np.sin(2*np.pi*f*t)
	    lollipop(t, a)
	    plt.gcf().set_size_inches((8, 4))
	    plt.grid(True)
	    plt.show()
	"""

	hold_status = plt.ishold()
	plt.hold(True)
	lolli_fmt = {'linewidth': lw, 'color': color}
	pop_fmt = {'markeredgecolor': color, 'markerfacecolor':'None', 'linestyle': 'None', \
	                'marker': 'o', 'markersize':10, 'markeredgewidth': lw*1.1}
	# Plot the circles
	plt.plot(x, y, **pop_fmt)

	# Make x and y row vectors, then plot as sticks
	x = x.transpose()
	y = y.transpose()
	x = np.vstack((x, x, float('NaN')*np.ones(x.shape)))
	y = np.vstack((y, repmat(ybot, 2, y.shape[0])))
	plt.plot(x, y, **lolli_fmt)

	plt.hold(hold_status)

def test_lollipop():
	"""Test function for lollipop()"""
	t = np.arange(1, 20)*1e-3
	f = 20.
	a = np.sin(2*np.pi*f*t)
	plt.figure()
	lollipop(t, a)
	plt.grid(True)



# -*- coding: utf-8 -*-
# _rmsGain.py
# Module providing the rmsGain function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the rmsGain() function
"""

import numpy as np
from scipy.linalg import norm
from ._evalTF import evalTF

def rmsGain(H, f1, f2, N=100):
	"""Compute the root mean-square gain of the discrete-time
	TF ``H`` in the frequency band ``(f1, f2)``.
	"""

	w = np.linspace(2*np.pi*f1, 2*np.pi*f2, N)
	g = norm(evalTF(H, np.exp(1j*w))) / np.sqrt(N)

	return g
	
def test_rmsGain():
	"""Test function for rmsGain()
	"""
	from ._utils import empty
	H = empty()
	H.num = (1,)
	H.den = (1, 2, 10)
	f1 = 0.001
	f2 = 0.5
	res = rmsGain(H, f1, f2, N=1000)
	res1 = 0.102245275091
	assert np.allclose((res,), (res1,), rtol=1e-05, atol=1e-08)




# -*- coding: utf-8 -*-
# _utils.py
# Miscellaneous functions and stdlib wrappers for MATLAB functions
# that do not find a direct replacement in numpy/scipy.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

""" Miscellaneous functions and wrappers for MATLAB functions
 that do not find a direct replacement in numpy/scipy.
"""

from warnings import warn
import collections, fractions
from fractions import Fraction as Fr
import numpy as np
from scipy.signal import lti, ss2tf, ss2zpk, zpk2tf

from._constants import eps
from ._partitionABCD import partitionABCD

def rat(x, tol):
    """Rational fraction approximation.

    Calculate A and B such that:

    .. math::

      x = \\frac{A}{B} + \\epsilon

    where:

    .. math::

        |\\epsilon| < tol


    .. note:: A, B are of type 'int'
    """
    return Fr(float(x)).limit_denominator(int(1/float(tol))).numerator, \
                Fr(float(x)).limit_denominator(int(1/float(tol))).denominator

gcd = fractions.gcd

lcm = lambda a, b: int(a*b / float(gcd(a, b)))
lcm.__doc__ = """Calculate the Least Common Multiple of ``a`` and ``b``.\n"""

class empty:
    """An empty function used to hold attributes"""
    def __init__(self):
        pass

def mfloor(x):
    """Round ``x`` towards -Inf.

    This is a MATLAB-compatible floor function, numpy's ``floor()``
    behaves differently.

    If the elements of ``x`` are complex, real and imaginary parts are
    rounded separately.
    """
    iform = save_input_form(x)
    x = carray(x)
    def _mfloor(z):
        """Base function to generate the ufunc floor"""
        z = np.real_if_close(z)
        if np.iscomplex(z):
            return _mfloor(np.real(z)) + 1j*_mfloor(np.imag(z))
        return np.floor(z) if np.sign(z) >= 0 else -np.ceil(-z)
    _internal = np.frompyfunc(_mfloor, 1, 1)
    xf = np.array(_internal(x), dtype=x.dtype)
    return restore_input_form(xf, iform)

def zpk(z, p, k):
    """Returns a zpk object with the zeros (z), poles (p) and gain (k) provided.
    """
    t = empty()
    t.form = 'zp'
    t.zeros, t.poles, t.k = carray(z), carray(p), k
    return t

def carray(x):
    """Check that x is an ndarray. If not, try to convert it to ndarray.
    """
    if not hasattr(x, 'shape'):
        if not hasattr(x, '__len__'):
            x = np.array((x,))
        else:
            x = np.array(x)
    elif not len(x.shape):
        x = x.reshape((1,))
    else:
        pass #nothing to do here
    return x

def cplxpair(x, tol=100):
    """
    Sort complex numbers into complex conjugate pairs.

    This function replaces MATLAB's cplxpair for vectors.
    """
    x = carray(x)
    x = x.tolist()
    x = [np.real_if_close(i, tol) for i in x]
    xreal = np.array(list(filter(np.isreal, x)))
    xcomplex = np.array(list(filter(np.iscomplex, x)))
    xreal = np.sort_complex(xreal)
    xcomplex = np.sort_complex(xcomplex)
    xcomplex_ipos = xcomplex[xcomplex.imag >  0.]
    xcomplex_ineg = xcomplex[xcomplex.imag <= 0.]
    res = []
    for i, j in zip(xcomplex_ipos, xcomplex_ineg):
        if not abs(i - np.conj(j)) < tol*eps:
            raise ValueError("Complex numbers can't be paired.")
        res += [j, i]
    return np.hstack((np.array(res), xreal))

def minreal(tf, tol=None):
    """Remove pole/zero pairs from a transfer function
    when the two match within the tolerance tol.
    
    tf may be a transfer function (lti object) or a list of transfer functions
    tol is optional and defaults to the system epsilon if unset.
    
    return a list of tfs or a tf, depending on the input type.
    """
    # initially based on python-control
    # which is in turn based on octave minreal
    # then modified considerably

    # recursively handle multiple tfs
    if not _is_zpk(tf) and not _is_num_den(tf) and not _is_A_B_C_D(tf) \
    and (isinstance(tf, list) or isinstance(tf, tuple)):
        ret = []
        for tfi in tf:
            ret += [minreal(tfi, tol)]
        return ret

    # python-control support is there but not tested and not advertised.
    if hasattr(tf, 'returnScipySignalLti'):
        tf = tf.returnScipySignalLti()[0][0]

    # default accuracy
    sqrt_eps = np.sqrt(eps)
    
    if (hasattr(tf, 'inputs') and not tf.inputs == 1) or \
       (hasattr(tf, 'outputs') and not tf.outputs == 1):
        raise TypeError("Only SISO transfer functions can be evaluated.")
    if (hasattr(tf, 'zeros') and hasattr(tf, 'poles')):
        # LTI objects have poles and zeros, 
        zeros = tf.zeros
        poles = tf.poles
        if hasattr(tf, 'k'):
            k = tf.k
        elif hasattr(tf, 'gain'):
            k = tf.gain
    elif hasattr(tf, 'num') and hasattr(tf, 'den'):
        num = tf.num
        den = tf.den
        k = num[0]/den[0]
        zeros = np.roots(num)
        poles = np.roots(den)
    else:
        zeros, poles, k = _get_zpk(tf)
    
    zeros = carray(zeros)
    poles = carray(poles)
    zeros.sort()
    poles.sort()
    reducedzeros = []

    for z in zeros:
        t = tol or 1000 * max(eps, abs(z) * sqrt_eps)
        idx = np.where(abs(poles - z) < t)[0]
        if len(idx):
            # cancel this zero against one of the poles
            # remove the pole and do not add the zero to the new
            poles = np.delete(poles, idx[0])
        else:
            # no matching pole
            reducedzeros.append(z)
    if len(reducedzeros):
        newzeros = carray(reducedzeros)
        num = k * np.real(np.poly(newzeros))
    else:
        num = np.array([k])
    den = np.real(np.poly(poles))

    return lti(num, den)

def diagonal_indices(a, offset=0):
    """Return the indices to the main diagonal of a 2D array a (if offset = 0),
    or to a secondary diagonal, having the offset from the main one as specified.

    The array a does not need to be square.

    Note: the sup-diagonal is at offset +1, the sub-diagonal at offset -1.

    Returns: (xs, ys)
    """
    di, dj = np.diag_indices_from(a[:min(a.shape), :min(a.shape)])
    if offset > 0:
        di, dj = zip(*[(i, j) for i, j in zip(di, dj+offset) if 0 <= j < a.shape[1]])
    elif offset == 0:
        pass
    else:
        di, dj = zip(*[(i, j) for i, j in zip(di-offset, dj) if 0 <= i < a.shape[0]])
    return di, dj

def circshift(a, shift):
    """Shift an array circularly.

    The ``circshift(a, shift)`` function circularly shifts the values in the
    array ``a`` by ``shift`` elements.

    **Parameters:**

    a : ndarray
        the array to be shifted. Notice that a should have a greater or equal
        number of dimensions than 'shift' ('shift' being a scalar is equal to
        'shift' having one dimension.)

    shift : int or ndarray-like of int.
        the N-th element specifies the shift amount for the N-th dimension
        of the input array 'a'.

    If an element in 'shift' is positive, the values of A are
    shifted to higher-index rows (ie down) or to higher-index columns
    (ie to the right).

    If the element is negative, the values of A are shifted in the opposite
    directions, towards lower-index rows (ie up) or to lower-index columns
    (ie right).

    If shift is an int, the shift happens along axis=0.

    All dimensions that do not have a corresponding shift value in 'shift'
    are left untouched (ie shift=(1,0,0) is equal to shift=(1,), with the
    exception that the former will trigger an IndexError if a.ndim < 3).

    **Returns:**

    The shifted array.
    """
    if np.isscalar(shift):
        shift = [shift]
    for axis, ashift in enumerate(shift):
        a = np.roll(a, ashift, axis=axis)
    return a

def save_input_form(a):
    """Save the form of `a` so that it can be restored later on

    Returns: an object representing the form of `a`, to be passed to
                 restore_input_form(a, form)
    """
    if np.isscalar(a):
        ret = 'scalar'
    elif hasattr(a, 'shape'):
        ret = a.shape
    elif type(a) == tuple:
        ret = 'tuple'
    elif type(a) == list:
        ret = 'list'
    else:
        raise TypeError("Unsupported input %s" % repr(a))
    return ret

def restore_input_form(a, form):
    """Restore the form of `a` according to `form`.

    Returns: the object `a`, in the correct `form`.

        Note: use `save_input_form(a)` to get the object `form`
    """
    if form == 'scalar':
        a = np.real_if_close(a)
        if not np.isscalar(a):
            a = a.reshape((1, ))[0]
    elif form == 'tuple':
        if not type(a) == tuple:
            a = [np.real_if_close(i).reshape((1,))[0] for i in a]
            a = tuple(a)
    elif form == 'list':
        if not type(a) == list:
            a = [np.real_if_close(i).reshape((1,))[0] for i in a]
            a = list(a)
    else:
        a = a.reshape(form)
    return a

def pretty_lti(arg):
    """Given the lti object ``arg`` return a *pretty* representation."""
    z, p, k = _get_zpk(arg)
    z = np.atleast_1d(z)
    p = np.atleast_1d(p)
    if not len(z) and not len(p):
        return "%g" % k
    ppstr = ["", "", ""]
    if k != 1:
        ppstr[1] = "%g " % k
    for i, s in zip((0, 2), (z, p)):
        rz = None
        for zi in cplxpair(s)[::-1]:
            zi = np.round(np.real_if_close(zi), 4)
            if np.isreal(zi):
                ppstr[i] += "(z %+g) " % -zi
            else:
                if rz is None:
                    rz = zi
                    continue
                else:
                    ppstr[i] += "(z^2 %+gz %+g) " % \
                                (np.round(np.real_if_close(-rz - zi), 3), 
                                 np.round(np.real_if_close(rz*zi), 4))
                    rz = None
        ppstr[i] = ppstr[i][:-1] if len(ppstr[i]) else "1"
    if ppstr[2] == '1':
        return ppstr[1] + ppstr[0]
    else:
        if ppstr[0] == '1' and len(ppstr[1]) and float(ppstr[1]) != 1.:
            ppstr[0] = ppstr[1][:-1]
            ppstr[1] = ""
        ppstr[1] += "-"*(max(len(ppstr[0]), len(ppstr[2])) + 2)
        ppstr[0] = ppstr[0].center(len(ppstr[1]))
        ppstr[2] = ppstr[2].center(len(ppstr[1]))
        return "\n".join(ppstr)

def _get_zpk(arg, input=0):
    """Utility method to convert the input arg to a z, p, k representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    input : scalar
        In case the system has multiple inputs, which input is to be
        considered. Input `0` means first input, and so on.

    **Returns:**

    The sequence of ndarrays z, p and a scalar k

    **Raises:**

    TypeError, ValueError

    .. warn: support for MISO transfer functions is experimental.
    """
    z, p, k = None, None, None
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
        z, p, k = ss2zpk(A, B, C, D, input=input)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
        z, p, k = arg.zeros, arg.poles, arg.gain
    elif _is_zpk(arg):
        z, p, k = carray(arg[0]).squeeze(), carray(arg[1]).squeeze(), arg[2]
    elif _is_num_den(arg):
        sys = lti(*arg)
        z, p, k = sys.zeros, sys.poles, sys.gain
    elif _is_A_B_C_D(arg):
        z, p, k = ss2zpk(*arg, input=input)
    elif isinstance(arg, collections.Iterable):
        ri = 0
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched lti representations.
            if hasattr(i, 'B'):
                iis = i.B.shape[1]
                if input < ri + iis:
                    z, p, k = ss2zpk(i.A, i.B, i.C, i.D, input=input-ri)
                    break
                else:
                    ri += iis
                    continue
            elif _is_A_B_C_D(arg):
                iis = arg[1].shape[1]
                if input < ri + iis:
                    z, p, k = ss2zpk(*arg, input=input-ri)
                    break
                else:
                    ri += iis
                    continue
            else:
                if ri == input:
                    sys = lti(*i)
                    z, p, k = sys.zeros, sys.poles, sys.gain
                    break
                else:
                    ri += 1
                    continue
                ri += 1
        if (z, p, k) == (None, None, None):
           raise ValueError("The LTI representation does not have enough" + \
                            "inputs: max %d, looking for input %d" % \
                            (ri - 1, input))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)
    return z, p, k

def _get_num_den(arg, input=0):
    """Utility method to convert the input arg to a (num, den) representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    input : scalar
        In case the system has multiple inputs, which input is to be
        considered. Input `0` means first input, and so on.

    **Returns:**

    The sequence of ndarrays num, den

    **Raises:**

    TypeError, ValueError

    .. warn: support for MISO transfer functions is experimental.
    """
    num, den = None, None
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
        num, den = ss2tf(A, B, C, D, input=input)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
        num, den = arg.num, arg.den
    elif _is_num_den(arg):
        num, den = carray(arg[0]).squeeze(), carray(arg[1]).squeeze()
    elif _is_zpk(arg):
        num, den = zpk2tf(*arg)
    elif _is_A_B_C_D(arg):
        num, den = ss2tf(*arg, input=input)
    elif isinstance(arg, collections.Iterable):
        ri = 0
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched representations.
            if hasattr(i, 'B'): #lti
                iis = i.B.shape[1]
                if input < ri + iis:
                    num, den = ss2tf(i.A, i.B, i.C, i.D, input=input-ri)
                    break
                else:
                    ri += iis
            else:
                sys = lti(*i)
                iis = sys.B.shape[1]
                if input < ri + iis:
                    num, den = ss2tf(sys.A, sys.B, sys.C, sys.D, input=input-ri)
                    break
                else:
                    ri += iis

        if (num, den) == (None, None):
           raise ValueError("The LTI representation does not have enough" + \
                            "inputs: max %d, looking for input %d" % \
                            (ri - 1, input))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)

    if len(num.shape) > 1:
        num = num.squeeze()
    if len(den.shape) > 1:
        num = den.squeeze()

    # default accuracy: sqrt_ps
    sqrt_eps = np.sqrt(eps)
    while True:
        if abs(num[0]) < sqrt_eps:
            num = num[1:]
        else:
            break
    while True:
        if abs(den[0]) < sqrt_eps:
            den = den[1:]
        else:
            break

    return num, den

def _getABCD(arg):
    """Utility method to convert the input arg to an A, B, C, D representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    **Returns:**

    The sequence of ndarrays A, B, C, D

    **Raises:**

    TypeError, ValueError
    """
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
       A, B, C, D = arg.A, arg.B, arg.C, np.atleast_2d(arg.D)
    elif _is_zpk(arg) or _is_num_den(arg) or _is_A_B_C_D(arg):
       sys = lti(*arg)
       A, B, C, D = sys.A, sys.B, sys.C, sys.D
    elif isinstance(arg, collections.Iterable):
        A, B, C, D = None, None, None, None
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched lti representations.
            sys = lti(*i) if not hasattr(i, 'A') else i
            if A is None:
                A = sys.A
            elif not np.allclose(sys.A, A, atol=1e-8, rtol=1e-5):
                raise ValueError("Mismatched lti list, A matrix disagreement.")
            else:
                pass
            if B is None:
                B = sys.B
            else:
                B = np.hstack((B, sys.B))
            if C is None:
                C = sys.C
            elif not np.allclose(sys.C, C, atol=1e-8, rtol=1e-5):
                raise ValueError("Mismatched lti list, C matrix disagreement.")
            if D is None:
                D = sys.D
            else:
                D = np.hstack((D, sys.D))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)
    return A, B, C, D

def _is_zpk(arg):
    """Can the argument be safely assumed to be a zpk tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 3 and \
       isinstance(arg[0], collections.Iterable) and \
       isinstance(arg[1], collections.Iterable) and np.isscalar(arg[2])

def _is_num_den(arg):
    """Can the argument be safely assumed to be a num, den tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 2 and \
           isinstance(arg[0], collections.Iterable) and \
           isinstance(arg[1], collections.Iterable)

def _is_A_B_C_D(arg):
    """Can the argument be safely assumed to be an (A, B, C, D) tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 4 and \
       (isinstance(arg[0], collections.Iterable) or np.is_scalar(arg[0])) and \
       (isinstance(arg[1], collections.Iterable) or np.is_scalar(arg[1])) and \
       (isinstance(arg[2], collections.Iterable) or np.is_scalar(arg[2])) and \
       (isinstance(arg[3], collections.Iterable) or np.is_scalar(arg[3]))

def _cell_like_list(shape, init=None):
    """Returns a list of lists (possibly of lists... etc...),
    with all values initialized to `init`.

    **Parameters:**

    shape: tuple of ints,
           the dimensions of the desidered object

    init: object, optional,
          the initialization value for every element in the object

    **Returns:**

    cell, a list of lists (...)
    """
    a = []
    for i in range(shape[0]):
        if len(shape) == 1:
            a.append(init)
        else:
            a.append(_cell_like_list(shape[1:], init))
    return a

def test_rat():
    """Test function for rat()"""
    import numpy.random as rnd
    for _ in range(10):
        n, d = rnd.randint(1, 5000), rnd.randint(1, 5000)
        fr = float(n)/float(d)
        nt, dt = rat(fr, tol=200e-6)
        assert np.allclose((n/float(d) -  nt/float(dt),), (0.,), atol=200e-6, rtol=1e-12)

def test_gcd_lcm():
    """Test function for gcd() and lcm"""
    a, b = 36, 31721
    tlcm, tgcd = 1141956, 1
    assert lcm(a, b) == tlcm
    assert gcd(a, b) == tgcd

def test_mfloor():
    """Test function for mfloor()"""
    tv = np.linspace(-1, 1, 10)
    tres = np.zeros(tv.shape)
    tres[tv >= 0] = np.floor(tv[tv>=0])
    tres[tv <  0] = -np.ceil(np.abs(tv[tv<0]))
    tresf = mfloor(tv)
    assert np.allclose(tres, tresf, atol=1e-8, rtol=1e-5)
    # w complex values.
    tv = [-1.9,  -0.2,  3.4,  5.6,  7.0,  2.4+3.6j]
    tres = [-2.0, -1.0, 3.0, 5.0, 7.0, (2+3j)]
    assert mfloor(tv) == tres

def test_carray():
    """Test function for carray()"""
    # don't touch 1d arrays
    a = np.arange(10)
    b = a.copy()
    assert np.all(carray(a) == b)
    # reshape 0-d arrays
    a = np.array(1)
    b = a.reshape((-1,))
    assert np.all(carray(a) == b)
    # convert lists and tuples to 1d arrays
    a = np.arange(10)
    c = a.tolist()
    b = tuple(c)
    assert np.all(a == carray(b))
    assert np.all(a == carray(c))
    # convert scalars to 1d arrays
    a = np.array((1,))
    b = 1
    assert np.all(carray(b) == a)

def test_zpk():
    """Test function for zpk()
    """
    z = [2,]
    p = [1, 3]
    k = 1
    t = zpk(z, p, k)
    assert t.form == 'zp'
    assert t.zeros.tolist() == z
    assert t.poles.tolist() == p
    assert t.k == k

def test_cplxpair():
    """Test function for cplxpair()
    """
    a = np.array([1 + eps*20j, 1.1 + 2j, 1.1 - (2+50*eps)*1j, .1 + (1+99*eps)*.2j, .1 - .2j])
    assert np.allclose(cplxpair(a), np.array([0.1-0.2j, 0.1+0.2j, 1.1-2.j, 1.1+2.j, 1.0+0.j]),
                       atol=100*eps)

def test_diagonal_indices():
    """Test function for diagonal_indices()
    """
    a = np.arange(1, 26)
    a = a.reshape((5, 5))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 7, 13, 19, 25))
    assert np.allclose(dp1, (2, 8, 14, 20))
    assert np.allclose(dm1, (6, 12, 18, 24))
    a = np.arange(1, 21)
    a = a.reshape((4, 5))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 7, 13, 19))
    assert np.allclose(dp1, (2, 8, 14, 20))
    assert np.allclose(dm1, (6, 12, 18))
    a = a.reshape((5, 4))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 6, 11, 16))
    assert np.allclose(dp1, (2, 7, 12))
    assert np.allclose(dm1, (5, 10, 15, 20))

def test_circshift():
    """Test function for circshift()
    """
    A = np.arange(1, 10).reshape((3, 3))
    shift = np.array((1, -1))
    Ashifted = circshift(A, shift)
    Ares = np.array(((8, 9, 7), (2, 3, 1), (5, 6, 4)))
    assert np.allclose(Ashifted, Ares)

def test_save_input_form():
    """Test function for save_input_form() and restore_input_form()"""
    a1 = np.arange(1, 26)
    a2 = a1.reshape((1, -1))
    form_a1 = save_input_form(a1)
    assert a1.shape == restore_input_form(a2, form_a1).shape
    assert np.allclose(a1, restore_input_form(a2, form_a1))
    b1 = 5
    b2 = carray(b1)
    form_b1 = save_input_form(b1)
    assert np.isscalar(restore_input_form(b2, form_b1))
    assert b1 == restore_input_form(b2, form_b1)

def test_get_zpk():
    """Test function for _get_zpk()"""
    from scipy.signal import zpk2ss
    z = (.4,)
    p = (.9, .1 + .2j, .1 -.2j)
    k = .4
    zt, pt, kt = _get_zpk(zpk2ss(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)
    zt, pt, kt = _get_zpk(zpk2tf(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)
    zt, pt, kt = _get_zpk(lti(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)

def test_get_num_den():
    """Test function for _get_num_den()"""
    from scipy.signal import zpk2ss, zpk2tf
    z = (.4,)
    p = (.9, .1 + .2j, .1 -.2j)
    k = .4
    num, den = zpk2tf(z, p, k)
    numt, dent = _get_num_den((num, den)) #num, den
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    numt, dent = _get_num_den((z, p, k)) #zpk
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    numt, dent = _get_num_den(lti(z, p, k)) #LTI
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    A, B, C, D = zpk2ss(z, p, k) #A,B,C,D
    D = np.atleast_2d(D)
    numt, dent = _get_num_den((A, B, C, D)) #A,B,C,D
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    ABCD = np.vstack((
                      np.hstack((A, B)),
                      np.hstack((C, D))
                    ))
    numt, dent = _get_num_den(ABCD) #A,B,C,D
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)

def test_minreal():
    """Test function for minreal()"""
    from scipy.signal import zpk2ss
    z = np.array([0, 1])
    p = np.array([1, .5 + .5j, .5 -.5j])
    k = 1
    zt = z[:-1]
    pt = p[1:]
    pt.sort()
    inobj = (z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = zpk2tf(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = zpk2ss(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = lti(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)

def test_pretty_lti():
    """Test function for pretty_lti()"""
    from ._synthesizeNTF import synthesizeNTF
    H = synthesizeNTF(5, 32, 1)
    tv = pretty_lti(H)
    res = '     (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)      \n' + \
          '--------------------------------------------------------\n' + \
          ' (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665) '
    assert res == tv
    assert int(pretty_lti(([], [], 2))) == 2
    assert pretty_lti([[1], [], 2]) == '2 (z -1)'
    assert pretty_lti([[], [.22222222], 2]) == \
           '      2      \n-------------\n (z -0.2222) '



# -*- coding: utf-8 -*-
# __init__.py
# python-deltasigma module init file
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""
python-deltasigma
=================

Introduction
------------

The **MATLAB Delta Sigma Toolbox** with **0% MATLAB** and **a *lot* more
Python**.

The **python-deltasigma** is a Python package to *synthesize, simulate,
scale and map to implementable structures* **delta sigma modulators**.

It aims to provide **a 1:1 Python port** of Richard Schreier's
***excellent*** `MATLAB Delta Sigma
Toolbox <http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox>`__,
the *de facto* standard tool for high-level delta sigma simulation, upon
which it is very heavily based.\ |githalytics.com alpha|

This project is a *work in progress*, not all functionality has been
ported, yet. The next figure shows the relationship between the main functions
and the avaliable functionality at a glance.

.. image:: ../doc/_static/functionality.png

To see the currently implemented functionality in action, take a look
at the following ipython notebooks:

-  `dsdemo1 <http://nbviewer.ipython.org/gist/ggventurini/8040189>`__,
   notebook port of the interactive ``dsdemo1.m``.
-  `dsdemo2 <http://nbviewer.ipython.org/gist/ggventurini/8044644>`__,
   notebook port of the interactive ``dsdemo2.m``.
-  `dsdemo4 <http://nbviewer.ipython.org/gist/ggventurini/8255785/dsdemo4.ipynb>`__,
   notebook port of ``dsdemo4.m``. `Audio
   file <https://gist.github.com/ggventurini/8255785/raw/8fb7d94236b917e6d557fb538d3f35a3144c038c/sax.wav.b64>`__.
-  `dsexample1 <http://nbviewer.ipython.org/7251113>`__, python
   version of ``dsexample1.m``.
-  `dsexample2 <http://nbviewer.ipython.org/8323435>`__, python
   version of ``dsexample2.m``.
-  `dsexample3 <http://nbviewer.ipython.org/8323046>`__, python
   version of ``dsexample3.m``.

They are also good for getting started quickly.

If you have some examples you would like to share, `send me a
mail <http://tinymailto.com/5310>`__, and I will add them to the above
list.

|Build Status| |Coverage Status|

For more detailed information about the development status take a look at 
`files.csv <https://github.com/ggventurini/python-deltasigma/blob/master/files.csv>`__.

The further functionality is expected to be ported according to `the
ROADMAP <https://github.com/ggventurini/python-deltasigma/blob/master/ROADMAP.md>`__.

Install
-------

Supported platforms
~~~~~~~~~~~~~~~~~~~

python-deltasigma runs on every platform and arch. supported by its
dependencies:

-  *Platforms*: Linux, Mac OS X, Windows.

-  *Archs*: x86, x86\_64 and armf (arm with floating point unit).

Dependencies
~~~~~~~~~~~~

Using python-deltasigma requires **Python 2** or **3**, **numpy**,
**scipy** (>= 0.11.0) and **matplotlib**.

They are packaged by virtually all the major *Linux distributions*.

On a Debian Linux system, you may install them issuing:

::

     # aptitude install python python-numpy python-scipy python-matplotlib

Refer to your system documentation for more information.

On *Windows*, I hear good things about:

-  `Enthought Canopy <https://www.enthought.com/store/>`__, a Python
   distribution that carries both free and commercial versions, and

-  `Anaconda <https://store.continuum.io/cshop/anaconda/>`__, which
   offers its full version for free.

I do not run Windows, so I can't really provide more info (sorry),
except that people tell me they manage to have a working setup.

*Mac OS X* is also supported by `Enthought
Canopy <https://www.enthought.com/store/>`__ and
`Anaconda <https://store.continuum.io/cshop/anaconda/>`__, which likely
provide the easiest and fastest solution to have a scientific Python
stack up and running.

More information can be found on the `scipy install
page <http://www.scipy.org/install.html>`__ and on the `matplotlib
homepage <http://matplotlib.org/>`__.

I wrote in a different context some directions to `compile numpy and
scipy
yourself <https://github.com/ahkab/ahkab/wiki/Install:-numpy-and-scipy>`__,
which also apply here. Be warned, it can easily get complicated.

Install python-deltasigma
~~~~~~~~~~~~~~~~~~~~~~~~~

You can install the latest stable version directly from the `Python Package
Index (PYPI) <http://pypi.python.org>`__, running::

    pip install deltasigma

The above command will also attempt to compile and install the dependencies -
for this to work you should already have the required libraries in place.

Otherwise, you can head over to `the Github repository 
<http://github.com/ggventurini/python-deltasigma>`__ and download the code
from there.

Then run::

    python setup.py install

The flag ``--local`` may be an interesting option to install the package for
the current user only, and it doesn't require root privileges.

Extras
~~~~~~

Building the documentation requires the
`sphinx <http://sphinx-doc.org/>`__ package. It is an optional step,
as the `the latest documentation is available
online <http://python-deltasigma.readthedocs.org/en/latest/>`__, without
need for you to build it.

If you plan to run the provided unit tests, then you should install
`setuptools <https://pypi.python.org/pypi/setuptools>`__, used to
access the reference function outputs. Testing *can* be automated with
`nose <https://pypi.python.org/pypi/nose/>`__, issuing::

    $ nosetests -v deltasigma/*.py

Documentation
-------------

1. You can find the included `package documentation
   online <http://python-deltasigma.readthedocs.org/en/latest/>`__.

2. The original MATLAB Toolbox provides in-depth documentation, which is
   very useful to understand what the toolbox is capable of. See
   `DSToolbox.pdf <https://github.com/ggventurini/python-deltasigma/blob/master/delsig/DSToolbox.pdf?raw=true>`__
   and
   `OnePageStory.pdf <https://github.com/ggventurini/python-deltasigma/blob/master/delsig/OnePageStory.pdf?raw=true>`__
   (*PDF warning*).

3. The book:

Richard Schreier, Gabor C. Temes, *Understanding Delta-Sigma Data
Converters*, ISBN: 978-0-471-46585-0, November 2004, Wiley-IEEE Press

is probably *the most authoritative resource on the topic*. Chapter 8-9
show how to use the MATLAB toolkit and the observations apply also to
this Python port. Links `on
amazon <http://www.amazon.com/Understanding-Delta-Sigma-Converters-Richard-Schreier/dp/0471465852>`__,
`on the Wiley-IEEE
press <http://eu.wiley.com/WileyCDA/WileyTitle/productCd-0471465852,miniSiteCd-IEEE2.html>`__.

*I am not affiliated with neither the sellers nor the authors.*

How to contribute
-----------------

Pull requests are welcome!
~~~~~~~~~~~~~~~~~~~~~~~~~~

If you want to port some code, fix a bug, add a cool example or implement new
functionality (in this case it may be a good idea to get in touch early on),
*that's awesome!*

There are only a few *guidelines*, which can be overridden every time it
is reasonable to do so:

-  Please try to follow ``PEP8``.

-  Try to keep the functions signature identical. Parameters with
   ``NaN`` default values have their default value replaced with
   ``None``.

-  If a function has a varible number of return values, its Python port
   should implement the maximum number of return values.

Reporting bugs
~~~~~~~~~~~~~~

What bugs, *there are no bugs!*

Jokes aside, please report all bugs on `on the Github issue tracker <https://github.com/ggventurini/python-deltasigma/issues>`__.

Support python-deltasigma
~~~~~~~~~~~~~~~~~~~~~~~~~

*I do not want your money.* I develop this software because I enjoy it
and because I use it myself.

If you wish to support the development of ``python-deltasigma`` or you
find the package useful or you otherwise wish to contribute monetarily,
***please donate to cancer research instead:***

-  `Association for International Cancer Research
   (eng) <http://www.aicr.org.uk/donate.aspx>`__, or
-  `Fond. IRCCS Istituto Nazionale dei Tumori
   (it) <http://www.istitutotumori.mi.it/modules.php?name=Content&pa=showpage&pid=24>`__.

Consider `sending me a mail <http://tinymailto.com/5310>`__ afterwards,
***it makes for great motivation!***

License, copyright, rationale and credits
-----------------------------------------

Why this project was born
~~~~~~~~~~~~~~~~~~~~~~~~~

I like challenges, Delta Sigma modulation and I don't have the money for
my own MATLAB license.

With this Python package you can simulate Delta Sigma modulators for
free, on any PC.

I hope you find it useful.

Licensing and copyright notice
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

All original MATLAB code is Copyright (c) 2009, Richard Schreier. See
the LICENSE file for the licensing terms.

The Python code here provided is a derivative work from the above
toolkit and subject to the same license terms.

This package contains some source code from ``pydsm``, also based on the
same MATLAB toolbox. The ``pydsm`` package is copyright (c) 2012, Sergio
Callegari.

When not otherwise specified, the Python code is Copyright 2013,
Giuseppe Venturini and the python-deltasigma contributors.

MATLAB is a registered trademark of The MathWorks, Inc.

.. |githalytics.com alpha| image:: https://cruel-carlota.pagodabox.com/36f25accf60f391456efe66910bf84f8
   :target: http://githalytics.com/ggventurini/python-deltasigma
   :width: 1
.. |Build Status| image:: https://travis-ci.org/ggventurini/python-deltasigma.png?branch=master
   :target: https://travis-ci.org/ggventurini/python-deltasigma
   :width: 86
.. |Coverage Status| image:: https://coveralls.io/repos/ggventurini/python-deltasigma/badge.png?branch=master
   :target: https://coveralls.io/r/ggventurini/python-deltasigma?branch=master
   :width: 86

Credits
~~~~~~~

The ``python-deltasigma`` package was written by 
`Giuseppe Venturini <https://github.com/ggventurini>`__, as a derivative work
of Richard Schreier's MATLAB Delta Sigma toolbox. It contains code from 
``pydsm``, also based on the same MATLAB toolbox and written by Sergio
Callegari.

Contributors: *future contributors will be added here.*

Implementation model
--------------------

The internal implementation of delta sigma modulators follows closely the one in
Richard Schreier's `MATLAB Delta Sigma
Toolbox <http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox>`__,
upon which the following documentation is very heavily based.


Modulator model
~~~~~~~~~~~~~~~

A delta-sigma modulator with a single quantizer is assumed to consist of 
quantizer connected to a loop filter as shown in the diagram below.

.. image:: ../doc/_static/modulator_model.png
    :align: center
    :alt: Modulator model

The loop filter
:::::::::::::::

The loop filter is described by an :math:`ABCD` matrix. For single-quantizer 
systems, the loop filter is a two-input, one-output linear system and 
:math:`ABCD` is an :math:`(n+1, n+2)` matrix, partitioned into
:math:`A` :math:`(n, n)`, :math:`B` :math:`(n, 2)`, :math:`C` :math:`(1, n)`
and :math:`D` :math:`(1, 2)` sub-matrices as shown below:

.. math::

    ABCD = 
        \\left[
        \\begin{array}{c|c}
          A & B \\\\ \\hline
          C & D 
        \\end{array}
        \\right].

The equations for updating the state and computing the output of the loop filter are:

.. math::

    x(n + 1) = Ax(n) + B
        \\left[
        \\begin{array}{c}
          u(n) \\\\
          v(n)
        \\end{array}
        \\right]

.. math::

    y(n + 1) = Cx(n) + D
        \\left[
        \\begin{array}{c}
          u(n) \\\\
          v(n)
        \\end{array}
        \\right].

Where :math:`u(n)` is the input sequence and :math:`v(n)` is the modulator output sequence. 

This formulation is sufficiently general to encompass all single-quantizer modulators which
employ linear loop filters. The toolbox currently supports translation to/from an ABCD descrip-
tion and coefficients for the following topologies:

 * CIFB : Cascade-of-integrators, feedback form.
 * CIFF : Cascade-of-integrators, feedforward form.
 * CRFB : Cascade-of-resonators, feedback form.
 * CRFF : Cascade-of-resonators, feedforward form.
 * CRFBD : Cascade-of-resonators, feedback form, delaying quantizer.
 * CRFFD : Cascade-of-resonators, feedforward form, delaying quantizer
 * PFF : Parallel feed-forward
 * Stratos : A CIFF-like structure with non-delaying resonator feedbacks [*]_

.. [*] Contributed to the MATLAB delta sigma toolbox in 2007 by Jeff Gealow.

See :ref:`topologies-diagrams` for a block-level view of the different modulator structures.

Multi-input and multi-quantizer systems can also be described with an
ABCD matrix and the previous equation
will still apply. For an :math:`n_i`-input, :math:`n_o`-output modulator,
the dimensions of the sub-matrices are 
:math:`A`: :math:`(n, n)`, :math:`B`: :math:`(n, n_i + n_o)`,
:math:`C`: :math:`(n_o, n)` and :math:`D`: :math:`(n_o, n_i+n_o)`.

Quantizer model
:::::::::::::::

The quantizer is ideal, producing integer outputs centered about zero. Quantizers with an even
number of levels are of the mid-rise type and produce outputs which are odd integers. Quantizers
with an odd number of levels are of the mid-tread type and produce outputs which are even inte-
gers.

.. image:: ../doc/_static/quantizer_model.png
    :align: center
    :alt: Quantizer model

.. seealso:: :func:`bquantize`, :func:`bunquantize`

.. _topologies-diagrams:

Topologies diagrams
~~~~~~~~~~~~~~~~~~~

CIFB
::::

.. image:: ../doc/_static/CIFB.png
    :align: center
    :alt: CIFB topology

CIFF
::::

.. image:: ../doc/_static/CIFF.png
    :align: center
    :alt: CIFF topology

CRFB
::::

.. image:: ../doc/_static/CRFB.png
    :align: center
    :alt: CRFB topology

CRFF
::::

.. image:: ../doc/_static/CRFF.png
    :align: center
    :alt: CRFF topology

CRFBD
:::::

.. image:: ../doc/_static/CRFBD.png
    :align: center
    :alt: CRFBD topology

CRFFD
:::::

.. image:: ../doc/_static/CRFFD.png
    :align: center
    :alt: CRFFD topology

Package contents
----------------

Key Functions
~~~~~~~~~~~~~

.. autosummary::
    :nosignatures:

    synthesizeNTF
    clans
    synthesizeChebyshevNTF
    simulateDSM
    simulateSNR
    realizeNTF
    stuffABCD
    mapABCD
    scaleABCD
    calculateTF
    realizeNTF_ct
    mapCtoD
    evalTFP

Functions for quadrature systems
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Notice the current version of ``python-deltasigma`` cannot synthesize
quadrature modulators. This feature is expected in v. 0.2.

Nontheless, the following functions are provided since if you already have
a synthesized modulator (or know its ABCD matrix), they allow you to simulate
the modulator with the standard tools for real ABCDs topologies.

.. autosummary::
    :nosignatures:

    mapQtoR
    mapRtoQ

Other selected functions
~~~~~~~~~~~~~~~~~~~~~~~~

The following are auxiliary functions that complement the key functions above.

.. autosummary::
    :nosignatures:

    mod1
    mod2
    calculateSNR
    predictSNR
    partitionABCD
    infnorm
    impL1
    l1norm
    pulse
    rmsGain

General utilities for data processing
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The following are generic functions useful for manipulating data,
conversions, padding and provide speciality functions which are
not otherwise available in the usual scientific Python stack.

.. autosummary::
    :nosignatures:

    db
    dbm
    dbp
    dbv
    undbm
    undbp
    undbv
    rms
    padb
    padl
    padr
    padt
    cplxpair
    mfloor
    rat
    gcd
    lcm

Plotting and data display utilitites
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Graphic functions:

.. autosummary::
    :nosignatures:

    plotPZ
    plotSpectrum
    figureMagic
    DocumentNTF
    PlotExampleSpectrum
    axisLabels
    bilogplot
    bplogsmooth
    lollipop
    changeFig

Textual and non-graphic functions

.. autosummary::
    :nosignatures:

    circ_smooth
    logsmooth
    pretty_lti
    SIunits

Utility functions for Delta Sigma simulation
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Functions for low-level handling of Delta Sigma modulator representations,
their evaluation and filtering.

.. autosummary::
    :nosignatures:

    bquantize
    bunquantize
    cancelPZ
    circshift
    delay
    ds_f1f2
    ds_freq
    ds_hann
    ds_optzeros
    ds_quantize
    ds_synNTFobj1
    dsclansNTF
    evalMixedTF
    evalRPoly
    evalTF
    nabsH
    peakSNR
    sinc_decimate
    zinc

All functions in alphabetical order
-----------------------------------

"""

__author__ = "Giuseppe Venturini and the python-deltasigma contributors"
__copyright__ = "Copyright 2013, Giuseppe Venturini"
__credits__ = ["Giuseppe Venturini"]
__license__ = "BSD 3-Clause License"
__version__ = '0.1rc3'
__maintainer__ = "Giuseppe Venturini"
__email__ = "ggventurini+github@gmail.com"
__status__ = "Stable"

# Package testing can be done remotely, without display. This would make
# matplotlib fail (and consequently, the test itself).
# We check for $DISPLAY, but this makes us probably lose in portability, 
# does Windows have this environment variable defined?
# Then again, who runs the test suit on a head-less windows machine...
# ... for the time being the following should be OK. If in the future that
# feature is needed by somebody, we can switch to
# if not os.system('python -c "import matplotlib.pyplot as plt;plt.figure()"')
import matplotlib, os
if not ('DISPLAY' in os.environ or os.environ.get('READTHEDOCS', None)):
    matplotlib.use('Agg')

from ._DocumentNTF import DocumentNTF
from ._PlotExampleSpectrum import PlotExampleSpectrum
from ._SIunits import SIunits
from ._axisLabels import axisLabels
from ._bilogplot import bilogplot
from ._bplogsmooth import bplogsmooth
from ._bquantize import bquantize
from ._bunquantize import bunquantize
from ._calculateSNR import calculateSNR
from ._calculateTF import calculateTF
from ._cancelPZ import cancelPZ
from ._changeFig import changeFig
from ._circ_smooth import circ_smooth
from ._clans import clans
from ._constants import eps
from ._db import db
from ._dbm import dbm
from ._dbp import dbp
from ._dbv import dbv
from ._delay import delay
from ._ds_f1f2 import ds_f1f2
from ._ds_freq import ds_freq
from ._ds_hann import ds_hann
from ._ds_optzeros import ds_optzeros
from ._ds_quantize import ds_quantize
from ._ds_synNTFobj1 import ds_synNTFobj1
from ._dsclansNTF import dsclansNTF
from ._evalMixedTF import evalMixedTF
from ._evalRPoly import evalRPoly
from ._evalF0 import evalF0
from ._evalF1 import evalF1
from ._evalTF import evalTF
from ._evalTFP import evalTFP
from ._figureMagic import figureMagic
from ._frespF1 import frespF1
from ._impL1 import impL1
from ._infnorm import infnorm
from ._l1norm import l1norm
from ._logsmooth import logsmooth
from ._lollipop import lollipop
from ._mapABCD import mapABCD
from ._mapCtoD import mapCtoD
from ._mapQtoR import mapQtoR
from ._mapRtoQ import mapRtoQ
from ._mod1 import mod1
from ._mod2 import mod2
from ._nabsH import nabsH
from ._padb import padb
from ._padl import padl
from ._padr import padr
from ._padt import padt
from ._partitionABCD import partitionABCD
from ._peakSNR import peakSNR
from ._plotPZ import plotPZ
from ._plotSpectrum import plotSpectrum
from ._predictSNR import predictSNR
from ._pulse import pulse
from ._realizeNTF import realizeNTF
from ._realizeNTF_ct import realizeNTF_ct
from ._rms import rms
from ._rmsGain import rmsGain
from ._scaleABCD import scaleABCD
from ._simulateDSM import simulateDSM
from ._simulateSNR import simulateSNR
from ._sinc_decimate import sinc_decimate
from ._stuffABCD import stuffABCD
from ._synthesizeChebyshevNTF import synthesizeChebyshevNTF
from ._synthesizeNTF import synthesizeNTF
from ._thermometer import thermometer
from ._undbm import undbm
from ._undbp import undbp
from ._undbv import undbv
from ._utils import circshift, cplxpair, mfloor, pretty_lti, rat, gcd, lcm
from ._zinc import zinc



# -*- coding: utf-8 -*-
# _partitionABCD.py
# Module providing the partitionABCD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the partitionABCD() function
"""

def partitionABCD(ABCD, m=None):
	"""Partition ABCD into A, B, C, D for an m-input state-space system."""
	# remember the ABCD matrix is assembled like this:
	# [[A, B], 
	#  [C, D]]
	if m is None:
		n = min(ABCD.shape) - 1
		m = ABCD.shape[1] - n
	else:
		n = ABCD.shape[1] - m

	r = ABCD.shape[0] - n

	A = ABCD[:n, :n]
	B = ABCD[:n, n:n+m]
	C = ABCD[n:n+r, :n]
	D = ABCD[n:n+r, n:n+m]
	return A, B, C, D

def test_partitionABCD():
	"""Test function for partitionABCD()"""
	# FIXME: no test for m-input systems
	import numpy as np
	from scipy.signal import lti
	ob = lti((1, ), (1, 2, 10))
	ab = np.hstack((ob.A, ob.B))
	cd = np.hstack((ob.C, ob.D.reshape((1,1))))
	abcd = np.vstack((ab, cd))
	a, b, c, d = partitionABCD(abcd)
	assert np.allclose(a, ob.A, rtol=1e-5, atol=1e-8)
	assert np.allclose(b, ob.B, rtol=1e-5, atol=1e-8)
	assert np.allclose(c, ob.C, rtol=1e-5, atol=1e-8)
	assert np.allclose(d, ob.D, rtol=1e-5, atol=1e-8)
	ABCD = [[1.000000000000000, 0., 0., 0.044408783846879, -0.044408783846879],
	        [0.999036450096481, 0.997109907515262, -0.005777399147297, 0., 0.499759089304780],
	        [0.499759089304780, 0.999036450096481, 0.997109907515262,  0., -0.260002096136488],
	        [0,                 0,                 1.000000000000000,  0, -0.796730400347216]]
	ABCD = np.array(ABCD)
	at = np.array([[1., 0., 0.],
	              [0.999036450096481, 0.997109907515262, -0.005777399147297],
	              [0.499759089304780, 0.999036450096481,  0.997109907515262]
	             ])
	bt = np.array([[0.044408783846879, -0.044408783846879],
	              [0., 0.499759089304780],
                      [0., -0.260002096136488]
                     ])
	ct = np.array([0., 0, 1])
	dt = np.array([0., -0.796730400347216])
	ar, br, cr, dr = partitionABCD(ABCD)
	assert np.allclose(at, ar, rtol=1e-5, atol=1e-8)
	assert np.allclose(bt, br, rtol=1e-5, atol=1e-8)
	assert np.allclose(ct, cr, rtol=1e-5, atol=1e-8)
	assert np.allclose(dt, dr, rtol=1e-5, atol=1e-8)

